"""
functions for detecting impact using point (Veh1) and edge (Veh2)
"""
import pandas as pd
import numpy as np
import math

def detect(vehicle_list, i, crush_data = None):
    #print('-- looking for impact --')
    #print(f'Vehicle 1 time = {vehicle_list[0].model.t[i]}')
    # TODO: assuming that vehicle_list = [veh1, veh2] - can fix by checking for impact point definition
    # current location of impact point - vehicle 1
    cgx1 = vehicle_list[0].model.Dx[i]  # take last value in Dx
    cgy1 = vehicle_list[0].model.Dy[i]  # take last value in Dy
    theta_rad1 = vehicle_list[0].model.theta_rad[i]

    impactp_gx = cgx1 + vehicle_list[0].pimpact_x * math.cos(theta_rad1) - vehicle_list[0].pimpact_y * math.sin(theta_rad1)
    impactp_gy = cgy1 + vehicle_list[0].pimpact_x * math.sin(theta_rad1) + vehicle_list[0].pimpact_y * math.cos(theta_rad1)

    # impact plane - vehicle 2
    cgx2 = vehicle_list[1].model.Dx[i]
    cgy2 = vehicle_list[1].model.Dy[i]
    theta_rad2 = vehicle_list[1].model.theta_rad[i]

    # impact point in veh2 reference frame
    impactp_veh2x = ((impactp_gx-cgx2) * math.cos(theta_rad2) + (impactp_gy-cgy2) * math.sin(theta_rad2))
    impactp_veh2y = (-1 * (impactp_gx-cgx2) * math.sin(theta_rad2) + (impactp_gy-cgy2) * math.cos(theta_rad2))

    """
    print(f'Vehicle 2 Heading Angle = {theta_rad2 * 180 / math.pi} degrees')
    print(f'Impact Point in Veh2 Frame x = {impactp_veh2x}, y = {impactp_veh2y}')
    print(f'Impact Point location: x = {impactp_gx}, y = {impactp_gy}')
    print(f'Veh1 CG location: x = {cgx1}, y = {cgy1}')
    print(f'Veh2 CG location: x = {cgx2}, y = {cgy2}')
    """

    """
    calculate relative position of impact point and impact edge
    save data to refer to determine relative velocity
    edge_loc is the location of the impact point along the impact edge,
    moving clockwise
    """
    if i == 0:
        crush_data = pd.DataFrame(np.nan, index=np.arange(len(vehicle_list[0].driver_input.t)),
                                columns = ['impact', 'edge_loc', 'normal_crush', 'impactp_veh2x', 'impactp_veh2y'])

    crush_data.impact[i] = False
    if vehicle_list[1].edgeimpact == 1:
        crush_data.normal_crush[i] = impactp_veh2x - vehicle_list[1].edgeimpact_x1
        crush_data.edge_loc[i] = vehicle_list[1].edgeimpact_y1 - impactp_veh2y
        if (crush_data.normal_crush[i] <= 0) & (impactp_veh2y >= vehicle_list[1].edgeimpact_y1) & (impactp_veh2y <= vehicle_list[1].edgeimpact_y2):
            crush_data.impact[i] = True
    elif vehicle_list[1].edgeimpact == 2:
        crush_data.normal_crush[i] = impactp_veh2y - vehicle_list[1].edgeimpact_y1
        crush_data.edge_loc[i] = vehicle_list[1].edgeimpact_x1 - impactp_veh2x
        if (crush_data.normal_crush[i] <= 0) & (impactp_veh2x <= vehicle_list[1].edgeimpact_x1) & (impactp_veh2x >= vehicle_list[1].edgeimpact_x2):
            crush_data.impact[i] = True
    elif vehicle_list[1].edgeimpact == 3:
        crush_data.normal_crush = vehicle_list[1].edgeimpact_x1 - impactp_veh2x
        crush_data.edge_loc[i] = vehicle_list[1].edgeimpact_y1 - impactp_veh2y
        if (crush_data.normal_crush <= 0) & (impactp_veh2y <= vehicle_list[1].edgeimpact_y1) & (impactp_veh2y >= vehicle_list[1].edgeimpact_y2):
            crush_data.impact[i] = True
    elif vehicle_list[1].edgeimpact == 4:
        crush_data.normal_crush[i] = vehicle_list[1].edgeimpact_y1 - impactp_veh2y
        crush_data.edge_loc[i] = vehicle_list[1].edgeimpact_x1 - impactp_veh2x
        if (crush_data.normal_crush[i] < 0) & (impactp_veh2x >= vehicle_list[1].edgeimpact_x1) & (impactp_veh2x <= vehicle_list[1].edgeimpact_x2):
            crush_data.impact[i] = True

    crush_data.impactp_veh2x[i] = impactp_veh2x
    crush_data.impactp_veh2y[i] = impactp_veh2y

    return crush_data



import matplotlib.pyplot as plt
import pandas as pd
import numpy as np
from .sdof_calcs.sdof_calculations import SingleDOFmodel
from copy import deepcopy
import inspect
import csv
import os

class SDOF_Model():
    """
    Inputs can be provided using a predefined dictionary "model_inputs"

    or inputs can be entered through user prompts
    if "None" is used for tstop model will run until vehicles seperate otherwise
    model will run to tstop if seperation has ocurred
    if a single value is used for model stiffness k, vehicle response will be
    calculated using a constant stiffness
    if a dataframe with 2 columns is entered [disp(ft) | force(lb)],
    vehicle response will be calculated using dataframe as a lookup table
    for force at a given displacement
    """

    def __init__(self, veh1, veh2, AB_offset = 0, model_inputs=None):
        self.type = "sdof"        # class type
        # create independent copy of vehicle class instances
        self.veh1 = deepcopy(veh1)
        self.veh2 = deepcopy(veh2)
        self.AB_offset = AB_offset     # <- [inches] adjust model crush to account for force required to initiate crush (offset = -A/B)
        # manually create inputs if not provided
        if (model_inputs == None):
            self.name = input('Enter name of SDOF model run:')
            self.cor = float(input('Provide a coefficient of restitution:'))
            self.k = input('Provide a single value or dataframe for mutual stiffness [lb/ft]:')
            self.tstop = float(input('Enter cut-off time (tstop (s)) to stop simulation or "None" to run simulation up to seperation:'))
        else:
            self.name = model_inputs['name']
            self.cor = model_inputs['cor']
            self.k = model_inputs['k']
            self.tstop = model_inputs['tstop']

        print("")
        print("------------ Model Inputs ---------------")
        print(f"Model Run = {self.name}")
        print(f"Coefficient of Restitution = {self.cor}")
        if (isinstance(self.k, int)) or (isinstance(self.k, float)):
            print(f"Constant Mutual Stiffness = {self.k} lb/ft ")
            self.ktype = 'constantK'               # define stiffness type for model
            if hasattr(self.veh1, 'k'):
                self.k1known = True   # if k1 is known, then veh1 crush can be calculated
            else:
                self.k1known = False

            if hasattr(self.veh2, 'k'):
                self.k2known = True
            else:
                self.k2known = False # if k2 is known, then veh2 crush can be calculated

        elif isinstance(self.k, pd.DataFrame):
            print(f"Stiffness Function Dataframe [disp (ft) | force (lb)] of shape = {self.k.shape}")
            self.ktype = 'tableK'                   # define stiffness type for model
            if hasattr(self.veh1, 'k'):
                self.k1known = True   # if k1 is known, then veh1 crush can be calculated
            else:
                self.k1known = False

            if hasattr(self.veh2, 'k'):
                self.k2known = True
            else:
                self.k2known = False # if k2 is known, then veh2 crush can be calculated

        if (isinstance(self.tstop, int)) or (isinstance(self.tstop, float)):
            print(f"Model will run until t = {self.tstop} seconds")
            self.ttype = 1                           # define t stop criteria
        elif (self.tstop == None):
            print("No stop time provided - model will run until vehicle seperation")
            self.ttype = 0                           # define t stop criteria
        else:
            print('Something other than a number or "None" used for stop time - model will run until vehicle seperation')
            self.ttype = 0                           # define t stop criteria

        # collect vehicle specific inputs

        print("")
        print("|------------ Vehicle Inputs -----------|")
        print("")
        print("<- Vehicle 1 ->")
        print("")
        try:
            if (isinstance(self.veh1.brake, int)) or (isinstance(self.veh1.brake, float)):
                print(f"{self.veh1.name} braking at {self.veh1.brake*100}%")
            else:
                print(f"{self.veh1.name} does not have a valid 'brake' entry")
                print(f"{self.veh1.name} - braking  set to 0 %")
                self.veh1.brake = 0
        except AttributeError:
            print(f"Braking not defined for {self.veh1.name} - value set to 0 %")
            self.veh1.brake = 0

        try:
            if (isinstance(self.veh1.vx_initial, int)) or (isinstance(self.veh1.vx_initial, float)):
                print(f"{self.veh1.name} initial speed is {self.veh1.vx_initial} mph")
            else:
                print(f"{self.veh1.name} does not have a valid initial forward velocity entry")
                print(f"{self.veh1.name} - velocity set to 0 mph")
                self.veh1.vx_initial = 0
        except AttributeError:
            print(f"Initial forward speed not defined for {self.veh1.name} - value set to 0 mph")
            self.veh1.vx_initial = 0

        print("")
        print("<- Vehicle 2 ->")
        print("")
        try:
            if (isinstance(self.veh2.brake, int)) or (isinstance(self.veh2.brake, float)):
                print(f"{self.veh2.name} braking at {self.veh2.brake*100}%")
            else:
                print(f"{self.veh2.name} does not have a valid 'brake' entry")
                print(f"{self.veh2.name} - braking  set to 0 %")
                self.veh2.brake = 0
        except AttributeError:
            print(f"Braking not defined for {self.veh2.name} - value set to 0 %")
            self.veh2.brake = 0

        try:
            if (isinstance(self.veh2.vx_initial, int)) or (isinstance(self.veh2.vx_initial, float)):
                print(f"{self.veh2.name} initial speed is {self.veh2.vx_initial} mph")
            else:
                print(f"{self.veh2.name} does not have a valid initial forward velocity entry")
                print(f"{self.veh2.name} - velocity set to 0 mph")
                self.veh2.vx_initial = 0
        except AttributeError:
            print(f"Initial forward speed not defined for {self.veh2.name} - value set to 0 mph")
            self.veh2.vx_initial = 0

        print(f"Model Closing Speed = {self.veh1.vx_initial - self.veh2.vx_initial} mph")
        print("")
        print("|----------- Input Complete ------------|")
        print("")

    #  Run SDOF Model
        self.model = SingleDOFmodel(W1 = self.veh1.weight,
                                    v1_initial = self.veh1.vx_initial,
                                    v1_brake = self.veh1.brake,
                                    W2 = self.veh2.weight,
                                    v2_initial = self.veh2.vx_initial,
                                    v2_brake = self.veh2.brake,
                                    k = self.k,
                                    cor = self.cor,
                                    tstop = self.tstop,
                                    ktype = self.ktype,
                                    ttype = self.ttype
                                    )
        if (self.AB_offset != 0):
            """The offset corrects for the force required to initiate permanent crush based on the A/B assumptions
            """
            print(f"Model Diplacement 'Dx' will be reduced by {self.AB_offset} inches")
            self.model.dx = [row + self.AB_offset / 12 for row in self.model.dx]

        if self.k1known:
            print("Vehicle 1 crush determined using provided stiffness")
            if self.ktype == "constantK":
                self.model['veh1_dx'] = [row * self.k / self.veh1.k for row in self.model.dx]

        if self.k2known:
            print("Vehicle 2 crush determined using provided stiffness")
            if self.ktype == "constantK":
                self.model['veh2_dx'] = [row * self.k / self.veh2.k for row in self.model.dx]

        # TODO: create attribute for vehicle inputs for saving / plotting run

    def input_dict(self):
        """
        return a dictionary to save / modify inputs
        """
        return {"name":self.name,
                "k":self.k,
                "cor":self.cor,
                "tstop":self.__ttype
                }

    def show(self):
        """ display all attributes assigned to the sdof model """
        for i in inspect.getmembers(self):
            if not i[0].startswith('_'):
                if not inspect.ismethod(i[1]):
                    print(i)

    def plot_fdx(self):
        """
        Plot force - mutual crush from model result
        """
        fig = plt.figure(figsize = (14,12))
        plt.title('Mutual Crush and Impact Force', fontsize=20)
        plt.plot(self.model.dx * -12, self.model.springF, label = f'V1={self.veh1.vx_initial} mph', color = "k")
        plt.xticks(fontsize=16)
        plt.yticks(fontsize=16)
        plt.xlim([0, round(-12 * min(self.model.dx) + 1)])
        plt.ylim([0, round(max(self.model.springF)+100)])
        ax = plt.gca()
        ax.spines['right'].set_visible(False)
        ax.spines['top'].set_visible(False)
        plt.xlabel('Mutual Crush (in)', fontsize=20)
        plt.ylabel('Force (lb)', fontsize=20)
        #plt.grid(which='both', axis='both')
        #plt.legend(fontsize=14, frameon = False)
        fig.show()

    def get_results(self):
        print('')
        print(f'Simulation results for {self.veh1.name}:')
        print(f'delta-V: {(self.model.v1.iloc[0]-self.model.v1.iloc[-1]) / 1.46667:0.2f} mph')
        print(f'Peak acceleration: {min(self.model.a1) /32.2:0.1f} g')
        if self.k1known:
            print(f'Residual crush: {self.model.veh1_dx.iloc[-1] * 12:0.2f} inches')

        print('')
        print(f'Simulation results for {self.veh2.name}:')
        print(f'delta-V: {(self.model.v2.iloc[-1]-self.model.v2.iloc[0]) / 1.46667:0.2f} mph')
        print(f'Peak acceleration: {max(self.model.a2) / 32.2:0.1f} g')
        if self.k2known:
            print(f'Residual crush: {self.model.veh2_dx.iloc[-1] * 12:0.2f} inches')



"""

"""
from .model_calcs.tire_model import tire_forces
import pandas as pd
import numpy as np
from scipy import integrate
import math
import csv
import os



# column list for vehicle model
column_list = ['t', 'vx','vy', 'Vx', 'Vy', 'Vr', 'oz_deg', 'oz_rad', 'delta_deg',
           'delta_rad', 'turn_rX', 'turn_rY', 'turn_rR', 'au', 'av',
           'ax','ay', 'ar', 'Ax', 'Ay', 'Ar', 'alphaz', 'alphaz_deg',
           'beta_deg','beta_rad', 'lf_fx', 'lf_fy', 'rf_fx', 'rf_fy',
           'rr_fx', 'rr_fy', 'lr_fx', 'lr_fy', 'lf_alpha', 'rf_alpha', 'rr_alpha', 'lr_alpha',
           'lf_lock', 'rf_lock', 'rr_lock', 'lr_lock', 'lf_fz', 'rf_fz', 'rr_fz', 'lr_fz',
           'theta_rad', 'theta_deg']

def vehicle_model(veh, sim_defaults):
    """
    Calculate vehicle dynamics from driver inputs and environmental inputs
    """
    # load defaults
    dt_motion = sim_defaults['dt_motion']  # iteration time step

    print(f"Vehicle motion will be simulated for {max(veh.driver_input.t)} seconds")
    veh.model = pd.DataFrame(np.nan, index=np.arange(len(veh.driver_input.t)), columns = column_list)

    veh.model.au[0] = 0         # no initial vehicle pitch
    veh.model.av[0] = 0         # no initial vehicle roll
    veh.model.vx[0] = veh.vx_initial * 1.46667  # convert input in mph to fps
    veh.model.vy[0] = veh.vy_initial * 1.46667  # convert input in mph to fps
    veh.model.theta_rad[0] = veh.head_angle * math.pi / 180   # initial heading angle
    veh.model.oz_rad[0] = veh.omega_z * (math.pi/180)  # initial angular rate (deg/s) - input
    veh.model.Vx[0] = veh.model.vx[0] * math.cos(veh.model.theta_rad[0]) - veh.model.vy[0] * math.sin(veh.model.theta_rad[0])
    veh.model.Vy[0] = veh.model.vx[0] * math.sin(veh.model.theta_rad[0]) + veh.model.vy[0] * math.cos(veh.model.theta_rad[0])

    for i in range(len(veh.driver_input.t)):
        veh.model.t[i] = round(i * dt_motion, 4)  # assigning time

        # get tire forces for t = 0
        veh = tire_forces(veh, i, sim_defaults)

        # local vehicle acceleration
        veh.model.au[i] = 32.2 / veh.weight * np.sum([veh.model.lf_fx[i],
                                                      veh.model.rf_fx[i],
                                                      veh.model.rr_fx[i],
                                                      veh.model.lr_fx[i]])

        veh.model.av[i] = 32.2 / veh.weight * np.sum([veh.model.lf_fy[i],
                                                      veh.model.rf_fy[i],
                                                      veh.model.rr_fy[i],
                                                      veh.model.lr_fy[i]])

        # rotation acceleration - alpha-z
        veh.model.alphaz[i] = (1 / veh.izz) * np.sum([veh.model.lf_fx[i] * veh.track / 2,
                                                      veh.model.lf_fy[i] * veh.lcgf,
                                                      -1 * veh.model.rf_fx[i] * veh.track / 2,
                                                      veh.model.rf_fy[i] * veh.lcgf,
                                                      -1 * veh.model.rr_fx[i] * veh.track / 2,
                                                      -1 * veh.model.rr_fy[i] * veh.lcgr,
                                                      veh.model.lr_fx[i] * veh.track / 2,
                                                      -1 * veh.model.lr_fy[i] * veh.lcgr])

        if i == 0:
            # vehicle acceleration in inertial frame
            veh.model.ax[i] = veh.model.au[i] + veh.model.oz_rad[i] * veh.model.vy[i]
            veh.model.ay[i] = veh.model.av[i] - veh.model.oz_rad[i] * veh.model.vx[i]
            veh.model.ar[i] = math.sqrt(veh.model.ax[i]**2 + veh.model.ay[i]**2)

            # inertial frame coordinates - capital letters
            veh.model.Ax[i] = veh.model.au[i] * math.cos(veh.model.theta_rad[i]) - veh.model.av[i] * math.sin(veh.model.theta_rad[i])
            veh.model.Ay[i] = veh.model.au[i] * math.sin(veh.model.theta_rad[i]) + veh.model.av[i] * math.cos(veh.model.theta_rad[i])

        if i > 0:
            # vehicle acceleration in inertial frame
            veh.model.ax[i] = veh.model.au[i] + veh.model.oz_rad[i-1] * veh.model.vy[i-1]
            veh.model.ay[i] = veh.model.av[i] - veh.model.oz_rad[i-1] * veh.model.vx[i-1]
            veh.model.ar[i] = math.sqrt(veh.model.ax[i]**2 + veh.model.ay[i]**2)

            # vehicle velocities
            veh.model.vx[i] = veh.model.vx[i-1] + dt_motion * np.mean([veh.model.ax[i-1], veh.model.ax[i]])
            veh.model.vy[i] = veh.model.vy[i-1] + dt_motion * np.mean([veh.model.ay[i-1], veh.model.ay[i]])

            # omega
            veh.model.oz_rad[i] = veh.model.oz_rad[i-1] + dt_motion * np.mean([veh.model.alphaz[i-1], veh.model.alphaz[i]])

            # heading angle
            veh.model.theta_rad[i] = veh.model.theta_rad[i-1] + dt_motion * np.mean([veh.model.oz_rad[i], veh.model.oz_rad[i-1]])

            # inertial frame coordinates - capital letters
            veh.model.Ax[i] = veh.model.au[i] * math.cos(veh.model.theta_rad[i]) - veh.model.av[i] * math.sin(veh.model.theta_rad[i])
            veh.model.Ay[i] = veh.model.au[i] * math.sin(veh.model.theta_rad[i]) + veh.model.av[i] * math.cos(veh.model.theta_rad[i])

            veh.model.Vx[i] = veh.model.Vx[i-1] + dt_motion * np.mean([veh.model.Ax[i-1], veh.model.Ax[i]])
            veh.model.Vy[i] = veh.model.Vy[i-1] + dt_motion * np.mean([veh.model.Ay[i-1], veh.model.Ay[i]])

        """

        """
        if veh.model.oz_rad[i] != 0:
            veh.model.turn_rX[i] = veh.model.Vy[i] / veh.model.oz_rad[i]    # turning radius in x direction
            veh.model.turn_rY[i] = veh.model.Vx[i] / veh.model.oz_rad[i]    # turning radius in y direction
            veh.model.turn_rR[i] = math.sqrt(veh.model.turn_rX[i]**2 + veh.model.turn_rY[i]**2)
        else:
            veh.model.turn_rX[i] = 0   # should actually be inf or undefined
            veh.model.turn_rY[i] = 0   # should actually be inf or undefined
            veh.model.turn_rR[i] = 0   # should actually be inf or undefined

        veh.model.Vr[i] = math.sqrt(veh.model.Vx[i]**2 + veh.model.Vy[i]**2)    # move to seperate calc
        veh.model.Ar[i] = math.sqrt(veh.model.Ax[i]**2 + veh.model.Ay[i]**2)

        # velocity vector in inertial frame
        veh.model.beta_rad[i] = math.atan2(veh.model.Vy[i], veh.model.Vx[i])    # move to seperate calc

    # vehicle position
    veh.model['Dx'] = veh.init_x_pos + integrate.cumtrapz(list(veh.model.Vx), list(veh.model.t), initial=0)
    veh.model['Dy'] = veh.init_y_pos + integrate.cumtrapz(list(veh.model.Vy), list(veh.model.t), initial=0)

    # converting to degrees
    # TODO: remove for speed
    veh.model.alphaz_deg = [row * 180 / math.pi for row in veh.model.alphaz]    # move to seperate calc
    veh.model.oz_deg = [row * 180 / math.pi for row in veh.model.oz_rad]        # move to seperate calc
    veh.model.theta_deg = [row * 180 / math.pi for row in veh.model.theta_rad]  # move to seperate calc
    veh.model.beta_deg = [row * 180 / math.pi for row in veh.model.beta_rad]    # move to seperate calc

    return veh



"""
Calculated vehicle premotion
Dependencies = tire model
Inputs - pulled from Vehicle class - initial speed (static), variable - braking / steering
Interpolates braking steering with time / distance
"""

import matplotlib.pyplot as plt
from .data.defaults.config import default_dict
from copy import deepcopy
from .multi_vehicle_model import multi_vehicle_model
from pycrash.model_calcs.position_data import position_data_motion, position_data_static
from .visualization.vehicle import plot_driver_inputs
from .visualization.initial_positions import initial_position
from .model_calcs.collision_plane import define_impact_plane, define_impact_edge
import pandas as pd
import numpy as np
import os

# load defaults
mu_max = default_dict['mu_max']  # maximum available friction
dt_motion = default_dict['dt_motion']  # iteration time step
dt_impact = default_dict['dt_impact']  # impact time step

figure_size = (16, 9)
xy_ratio = figure_size[0] / figure_size[1]

# look for Environment data, load if present
if os.path.isfile(os.path.join(os.getcwd(), "data", "input", "environment.csv")):
    enviro = pd.read_csv(os.path.join(os.getcwd(), "data", "input", "environment.csv"))
    if len(enviro) == 0:
        print('Environment file appears blank - no terrian data used')
        print(f'Constant friction {mu_max} used throughout')
    else:
        print('TODO - process terrain data')
else:
    print('No Environment File Provided')


class KinematicsTwo():
    """
    Generates vehicle motion based on inputs defined within Vehicle class
    No external forces
    Environment slope and bank is optionally defined as a function of X,Y components
    creates independent copy of vehicle at instantiation
    """

    def __init__(self, name, impact_type, veh1, veh2, mutual_stiffness=None, vehicle_friction=None, user_sim_defaults=None):
        self.name = name
        self.type = 'multimotion'  # class type for saving files
        self.veh1 = deepcopy(veh1)
        self.veh2 = deepcopy(veh2)

        if (impact_type not in ["SS", "IMPC"]):
            print("Not a valid impact type, choose SS, IMPC or SDOF")
            impact_type = input("Enter an impact type of SS or IMPC: ")

            if (impact_type not in ["SS", "IMPC"]):
                print("Not a valid impact type - value set to None")
                self.impact_type == None
            else:
                self.impact_type = impact_type
        else:
            self.impact_type = impact_type
            if vehicle_friction:
                self.veh_mu = vehicle_friction
            elif self.impact_type == 'SS':
                print(f"Vehicle friction for {self.name} is empty")
                self.veh_mu = float(input("Enter value for intervehicular friction: "))

        if impact_type == 'SS':
            if mutual_stiffness:
                self.kmutual = mutual_stiffness
            else:
                print(f"Mutual Stiffness for {self.name} is empty")
                self.kmutual = float(input("Enter value for mutual stiffness (lb/ft): "))
            if vehicle_friction:
                self.vehicle_mu = vehicle_friction
            else:
                print(f"Intervehicular friction for {self.name} is empty")
                self.vehicle_mu = float(input("Enter value for sliding friction: "))
        # load defaults
        if user_sim_defaults:
            # TODO: create check for user sim_defaults_input
            self.sim_defaults = user_sim_defaults
        else:
            self.sim_defaults = {'dt_motion': 0.01,
                                 'mu_max': 0.76,
                                 'alpha_max': 0.174533,  # 10 degrees
                                 'vehicle_mu': 0.3,
                                 'cor': 0.2}

        # check for driver inputs - Vehicle 1
        if isinstance(self.veh1.driver_input, pd.DataFrame):
            print(f"Driver input for {self.veh1.name} of shape = {self.veh1.driver_input.shape}")
        else:
            print(f'Driver input for {self.veh1.name} not provided - no braking or steering applied')
            print(f'Current driver input of type: {type(self.veh1.driver_input)}')
            end_time = int(input('Enter duration for simulation (seconds):'))
            t = list(np.arange(0, end_time + dt_motion, dt_motion))  # create time array from 0 to end time from user
            throttle = [0] * len(t)
            brake = [0] * len(t)
            steer = [0] * len(t)
            driver_input_dict = {'t': t, 'throttle': throttle, 'brake': brake, 'steer': steer}
            self.veh1.driver_input = pd.DataFrame.from_dict(driver_input_dict)
            print(f'Driver inputs for {self.veh1.name} set to zero for {end_time} seconds')

        # check for driver inputs - Vehicle 2
        if isinstance(self.veh2.driver_input, pd.DataFrame):
            print(f"Driver input for {self.veh2.name} of shape = {self.veh2.driver_input.shape}")
        else:
            print(f'Driver input for {self.veh2.name} not provided - no braking or steering applied')
            print(f'Current driver input of type: {type(self.veh2.driver_input)}')
            end_time = int(input('Enter duration for simulation (seconds):'))
            t = list(np.arange(0, end_time + dt_motion, dt_motion))  # create time array from 0 to end time from user
            throttle = [0] * len(t)
            brake = [0] * len(t)
            steer = [0] * len(t)
            driver_input_dict = {'t': t, 'throttle': throttle, 'brake': brake, 'steer': steer}
            self.veh2.driver_input = pd.DataFrame.from_dict(driver_input_dict)
            print(f'Driver inputs for {self.veh2.name} set to zero for {end_time} seconds')

        if self.impact_type == 'SS':
            if all(hasattr(self.veh1, attr) for attr in ["pimpact_x", "pimpact_y"]):
                print(f'Predefined impact point for {self.veh1.name}: x={self.veh1.pimpact_x:.1f}, y={self.veh1.pimpact_y:.1f}')
            else:
                print(f"Create impact point for {self.veh1.name} = striking vehicle")
                print("")
                self.veh1 = define_impact_plane(veh1, iplane=False)
        else:
            if all(hasattr(self.veh1, attr) for attr in ["pimpact_x", "pimpact_y", "impact_norm_rad"]):
                print(f'Predefined impact point for {self.veh1.name}: x={self.veh1.pimpact_x:.1f}, y={self.veh1.pimpact_y:.1f}, impact plane angle: {self.veh1.impact_norm_rad * 180 / 3.14159:.1f}')
            else:
                print(f"Create impact plane for {self.veh1.name} = striking vehicle")
                print("")
                self.veh1 = define_impact_plane(veh1)

        if all(hasattr(self.veh2, attr) for attr in ["edgeimpact_x1", "edgeimpact_y1", "edgeimpact_x2", "edgeimpact_y2", "edgeimpact"]):
            print(f'Predefined impact edge for {self.veh2.name} x1={self.veh2.edgeimpact_x1:.1f}, y1={self.veh2.edgeimpact_y1}; x2={self.veh2.edgeimpact_x2:.1f}, y2={self.veh2.edgeimpact_y2:.1f}')
        else:
            print(f"Create impacting edge for {self.veh2.name} = struck vehicle")
            print("")
            self.veh2 = define_impact_edge(veh2, iplane=False)

    def plot_inputs(self):
        for veh in [self.veh1, self.veh2]:
            plot_driver_inputs(veh)

    def plot_motion(self):
        for veh in [self.veh1, self.veh2]:
            fig, axs = plt.subplots(3, 2, figsize=figure_size, sharex='col')
            fig.suptitle(f'{veh.name}', fontsize=16)
            axs[0, 0].set_ylabel('Velocity (mph)', color='k')
            axs[0, 0].plot(self.model.t, self.model.Vx / 1.46667, color='k', label='Vx')
            axs[0, 0].plot(self.model.t, self.model.Vy / 1.46667, color='b', label='Vy')
            axs[0, 0].plot(self.model.t, self.model.Vr / 1.46667, color='g', linestyle='dashed', label='V')
            axs[0, 0].tick_params(axis='y', labelcolor='k')
            axs[0, 0].legend(frameon=False)

            axs[0, 1].set_ylabel('Acceleration (g)', color='k')
            axs[0, 1].tick_params(axis='y', labelcolor='k')
            axs[0, 1].plot(self.model.t, self.model.Ax / 32.2, color='k', label='Ax')
            axs[0, 1].plot(self.model.t, self.model.Ay / 32.2, color='b', label='Ay')
            axs[0, 1].plot(self.model.t, self.model.ax / 32.2, color='k', linestyle='dashed', label='ax')
            axs[0, 1].plot(self.model.t, self.model.ay / 32.2, color='b', linestyle='dashed', label='ay')
            axs[0, 1].legend(frameon=False)

            axs[1, 0].set_ylabel('Forward Tire Forces (lb)', color='k')
            axs[1, 0].tick_params(axis='y', labelcolor='k')
            axs[1, 0].plot(self.model.t, self.model.lf_fx, color='b', label='LF')
            axs[1, 0].plot(self.model.t, self.model.rf_fx, color='g', label='RF')
            axs[1, 0].plot(self.model.t, self.model.rr_fx, color='m', label='RR')
            axs[1, 0].plot(self.model.t, self.model.lr_fx, color='orange', label='LR')
            axs[1, 0].legend(frameon=False)

            axs[1, 1].set_ylabel('Rightward Tire Forces (lb)', color='k')
            axs[1, 1].tick_params(axis='y', labelcolor='k')
            axs[1, 1].plot(self.model.t, self.model.lf_fy, color='b', label='LF')
            axs[1, 1].plot(self.model.t, self.model.rf_fy, color='g', label='RF')
            axs[1, 1].plot(self.model.t, self.model.rr_fy, color='m', label='RR')
            axs[1, 1].plot(self.model.t, self.model.lr_fy, color='orange', label='LR')
            axs[1, 1].legend(frameon=False)

            axs[2, 0].set_xlabel('Time (s)')
            axs[2, 0].set_ylabel('Omega (deg/s), Alpha (deg/s/s)', color='k')
            axs[2, 0].tick_params(axis='y', labelcolor='k')
            axs[2, 0].plot(self.model.t, self.model.oz_deg, color='k', label='Omega')
            axs[2, 0].plot(self.model.t, self.model.alphaz_deg, color='r', label='Alpha')
            axs[2, 0].legend(frameon=False)

            axs[2, 1].set_xlabel('Time (s)')
            axs[2, 1].set_ylabel('Heading Angle (deg)', color='k')
            axs[2, 1].tick_params(axis='y', labelcolor='k')
            axs[2, 1].plot(self.veh_motion.t, self.veh_motion.theta_deg, color='k')

            plt.subplots_adjust(wspace=0.2, hspace=.1)
            plt.show()

    # plot initial positions and any motion data to show vehicle paths
    def show_initial_position(self, i=0):
        # TODO: future - can choose a time point
        [self.veh1, self.veh2] = position_data_static([self.veh1, self.veh2])
        initial_position(self.veh1, self.veh2, i)

    # run vehicle models iteratively to evaluate for impact
    def simulate(self, ignore_driver=False):
        if self.impact_type == 'SS':
            [self.veh1, self.veh2], self.crush_data = multi_vehicle_model(vehicle_list=[self.veh1, self.veh2], sim_defaults=self.sim_defaults,
                                                                          impact_type=self.impact_type, ignore_driver=ignore_driver, kmutual=self.kmutual, vehicle_mu=self.vehicle_mu)
        else:
            [self.veh1, self.veh2], self.crush_data = multi_vehicle_model(vehicle_list=[self.veh1, self.veh2], sim_defaults=self.sim_defaults,
                                                                          impact_type=self.impact_type, ignore_driver=ignore_driver)

        self.veh1 = position_data_motion(self.veh1, striking=True)
        self.veh2 = position_data_motion(self.veh2)

    def draw_simulation(self, i, tire_path=True):
        # create point data in vehicle and global frame
        """
        Plot Vehicle in Global reference frame
        """
        fig = plt.figure(figsize=figure_size)

        for veh in [self.veh1, self.veh2]:
            bdy_x = (veh.p_gx.b_lfc[i], veh.p_gx.b_rfc[i], veh.p_gx.b_rrc[i], veh.p_gx.b_lrc[i], veh.p_gx.b_lfc[i])
            bdy_y = (veh.p_gy.b_lfc[i], veh.p_gy.b_rfc[i], veh.p_gy.b_rrc[i], veh.p_gy.b_lrc[i], veh.p_gy.b_lfc[i])

            lfw_x = (veh.p_gx.lfw_a[i], veh.p_gx.lfw_b[i], veh.p_gx.lfw_c[i], veh.p_gx.lfw_d[i], veh.p_gx.lfw_a[i])
            lfw_y = (veh.p_gy.lfw_a[i], veh.p_gy.lfw_b[i], veh.p_gy.lfw_c[i], veh.p_gy.lfw_d[i], veh.p_gy.lfw_a[i])

            rfw_x = (veh.p_gx.rfw_a[i], veh.p_gx.rfw_b[i], veh.p_gx.rfw_c[i], veh.p_gx.rfw_d[i], veh.p_gx.rfw_a[i])
            rfw_y = (veh.p_gy.rfw_a[i], veh.p_gy.rfw_b[i], veh.p_gy.rfw_c[i], veh.p_gy.rfw_d[i], veh.p_gy.rfw_a[i])

            rrw_x = (veh.p_gx.rrw_a[i], veh.p_gx.rrw_b[i], veh.p_gx.rrw_c[i], veh.p_gx.rrw_d[i], veh.p_gx.rrw_a[i])
            rrw_y = (veh.p_gy.rrw_a[i], veh.p_gy.rrw_b[i], veh.p_gy.rrw_c[i], veh.p_gy.rrw_d[i], veh.p_gy.rrw_a[i])

            lrw_x = (veh.p_gx.lrw_a[i], veh.p_gx.lrw_b[i], veh.p_gx.lrw_c[i], veh.p_gx.lrw_d[i], veh.p_gx.lrw_a[i])
            lrw_y = (veh.p_gy.lrw_a[i], veh.p_gy.lrw_b[i], veh.p_gy.lrw_c[i], veh.p_gy.lrw_d[i], veh.p_gy.lrw_a[i])

            plt.plot(bdy_x, bdy_y, 'k')
            plt.scatter(veh.p_gx.lfw[i], veh.p_gy.lfw[i], c='b')  # left front wheel center
            plt.plot(lfw_x, lfw_y, 'b')
            plt.scatter(veh.p_gx.rfw[i], veh.p_gy.rfw[i], c='g')  # right front wheel center
            plt.plot(rfw_x, rfw_y, 'g')
            plt.scatter(veh.p_gx.rrw[i], veh.p_gy.rrw[i], c='m')  # right rear wheel center
            plt.plot(rrw_x, rrw_y, 'm')
            plt.scatter(veh.p_gx.lrw[i], veh.p_gy.lrw[i], c='orange')  # left rear wheel center
            plt.plot(lrw_x, lrw_y, 'orange')
            plt.scatter(veh.p_gx.cg[i], veh.p_gy.cg[i], s=100, c='k')  # vehicle CG

            if (veh.striking):  # striking vehicle
                plt.scatter(veh.p_gx.pimpact_x[i], veh.p_gy.pimpact_y[i], s=100, c='r')  # impact point
            # velocity vector
            plt.arrow(veh.p_gx.cg[i], veh.p_gy.cg[i], veh.p_gx.vel_v[i] - veh.p_gx.cg[i], veh.p_gy.vel_v[i] - veh.p_gy.cg[i], head_width=1, head_length=1, fc='r', ec='r')
            # vehicle axes
            plt.arrow(veh.p_gx.cg[i], veh.p_gy.cg[i], veh.p_gx.xaxis[i] - veh.p_gx.cg[i], veh.p_gy.xaxis[i] - veh.p_gy.cg[i], head_width=.5, head_length=0.5, fc='k', ec='k')
            plt.arrow(veh.p_gx.cg[i], veh.p_gy.cg[i], veh.p_gx.yaxis[i] - veh.p_gx.cg[i], veh.p_gy.yaxis[i] - veh.p_gy.cg[i], head_width=.5, head_length=0.5, fc='b', ec='b')

            # for loop up to i
            if (tire_path):
                for i in range(0, i):
                    if veh.p_gx.loc[i, 'lf_lock'] == 0:
                        plt.scatter(veh.p_gx.loc[i, 'lfw'], veh.p_gy.loc[i, 'lfw'], c='b', s=1, marker='.')
                    elif veh.p_gx.loc[i, 'lf_lock'] == 1:
                        plt.scatter(veh.p_gx.loc[i, 'lfw'], veh.p_gy.loc[i, 'lfw'], c='b', s=4, marker='s')

                    if veh.p_gx.loc[i, 'rf_lock'] == 0:
                        plt.scatter(veh.p_gx.loc[i, 'rfw'], veh.p_gy.loc[i, 'rfw'], c='g', s=1, marker='.')
                    elif veh.p_gx.loc[i, 'rf_lock'] == 1:
                        plt.scatter(veh.p_gx.loc[i, 'rfw'], veh.p_gy.loc[i, 'rfw'], c='g', s=4, marker='s')

                    if veh.p_gx.loc[i, 'rr_lock'] == 0:
                        plt.scatter(veh.p_gx.loc[i, 'rrw'], veh.p_gy.loc[i, 'rrw'], c='m', s=1, marker='.')
                    elif veh.p_gx.loc[i, 'rr_lock'] == 1:
                        plt.scatter(veh.p_gx.loc[i, 'rrw'], veh.p_gy.loc[i, 'rrw'], c='m', s=4, marker='s')

                    if veh.p_gx.loc[i, 'lr_lock'] == 0:
                        plt.scatter(veh.p_gx.loc[i, 'lrw'], veh.p_gy.loc[i, 'lrw'], c='orange', s=1, marker='.')
                    elif veh.p_gx.loc[i, 'lr_lock'] == 1:
                        plt.scatter(veh.p_gx.loc[i, 'lrw'], veh.p_gy.loc[i, 'lrw'], c='orange', s=4, marker='s')

        plt.gca().invert_yaxis()
        plt.show()



import plotly.graph_objects as go
import plotly.io as pio
pio.renderers.default = "browser"  # <- determines how plots are displayed using Plotly

width = 1900
aspect_ratio = 16 / 9
figure_size = (width, width / aspect_ratio)
wheel_colors = ['rgb(0, 0, 255)', 'rgb(0, 255, 0)', 'rgb(153, 0, 204)', 'rgb(255, 102, 0)']


# plot initial positions and any motion data to show vehicle paths
def initial_position(vehicle_list):
    # plot initial positions
    # grid grid based on initial position of vehicle
    # scale x,y axes accordingly
    # static geometry from model_calcs.position_data

    """
    combine two vehicles into a single df, to allow for max / min check
    """
    fig = go.Figure()
    for veh in vehicle_list:
        # Body and tire outlines
        bdy_x = (veh.Px['b_lfc'], veh.Px['b_rfc'], veh.Px['b_rrc'], veh.Px['b_lrc'], veh.Px['b_lfc'])
        bdy_y = (veh.Py['b_lfc'], veh.Py['b_rfc'], veh.Py['b_rrc'], veh.Py['b_lrc'], veh.Py['b_lfc'])

        lfw_x = (veh.Px['lfw_a'], veh.Px['lfw_b'], veh.Px['lfw_c'], veh.Px['lfw_d'], veh.Px['lfw_a'])
        lfw_y = (veh.Py['lfw_a'], veh.Py['lfw_b'], veh.Py['lfw_c'], veh.Py['lfw_d'], veh.Py['lfw_a'])

        rfw_x = (veh.Px['rfw_a'], veh.Px['rfw_b'], veh.Px['rfw_c'], veh.Px['rfw_d'], veh.Px['rfw_a'])
        rfw_y = (veh.Py['rfw_a'], veh.Py['rfw_b'], veh.Py['rfw_c'], veh.Py['rfw_d'], veh.Py['rfw_a'])

        rrw_x = (veh.Px['rrw_a'], veh.Px['rrw_b'], veh.Px['rrw_c'], veh.Px['rrw_d'], veh.Px['rrw_a'])
        rrw_y = (veh.Py['rrw_a'], veh.Py['rrw_b'], veh.Py['rrw_c'], veh.Py['rrw_d'], veh.Py['rrw_a'])

        lrw_x = (veh.Px['lrw_a'], veh.Px['lrw_b'], veh.Px['lrw_c'], veh.Px['lrw_d'], veh.Px['lrw_a'])
        lrw_y = (veh.Py['lrw_a'], veh.Py['lrw_b'], veh.Py['lrw_c'], veh.Py['lrw_d'], veh.Py['lrw_a'])

        # body outline
        fig.add_trace(go.Scatter(x=bdy_x, y=bdy_y,
                                 mode='lines',
                                 name=f'{veh.name} - body',
                                 line=dict(color='rgb(0, 0, 0)', width=2)))

        # wheel outline
        fig.add_trace(go.Scatter(x=lfw_x, y=lfw_y,
                                 mode='lines',
                                 name='LF',
                                 line=dict(color=wheel_colors[0], width=1)))
        fig.add_trace(go.Scatter(x=rfw_x, y=rfw_y,
                                 mode='lines',
                                 name='RF',
                                 line=dict(color=wheel_colors[1], width=1)))
        fig.add_trace(go.Scatter(x=rrw_x, y=rrw_y,
                                 mode='lines',
                                 name='RR',
                                 line=dict(color=wheel_colors[2], width=1)))
        fig.add_trace(go.Scatter(x=lrw_x, y=lrw_y,
                                 mode='lines',
                                 name='LR',
                                 line=dict(color=wheel_colors[3], width=1)))

        # CG
        cgx = [veh.init_x_pos, veh.init_x_pos]
        cgy = [veh.init_y_pos, veh.init_y_pos]

        fig.add_trace(go.Scatter(x=cgx, y=cgy,
                                 mode='markers',
                                 name=f'CG - {veh.name}',
                                 marker=dict(color='rgb(0, 0, 0)', size=10),
                                 ))

        # velocity vector
        fig.add_annotation(x=veh.Px['vel_v'],
                           y=veh.Py['vel_v'],
                           ax=veh.Px['cg'],
                           ay=veh.Py['cg'],
                           axref='x',
                           ayref='y',
                           xref='x',
                           yref='y',
                           text="",
                           showarrow=True,
                           arrowsize=2.5,
                           arrowhead=1,
                           arrowwidth=1.7,
                           arrowcolor='rgb(255, 0, 0)')

        # x axis
        fig.add_annotation(x=veh.Px['xaxis'],
                           y=veh.Py['xaxis'],
                           ax=veh.Px['cg'],
                           ay=veh.Py['cg'],
                           axref='x',
                           ayref='y',
                           xref='x',
                           yref='y',
                           text="",
                           showarrow=True,
                           arrowsize=2,
                           arrowhead=1,
                           arrowwidth=1.5,
                           arrowcolor='rgb(0, 0, 0)')

        # y axis
        fig.add_annotation(x=veh.Px['yaxis'],
                           y=veh.Py['yaxis'],
                           ax=veh.Px['cg'],
                           ay=veh.Py['cg'],
                           axref='x',
                           ayref='y',
                           xref='x',
                           yref='y',
                           text="",
                           showarrow=True,
                           arrowsize=2,
                           arrowhead=1,
                           arrowwidth=1.5,
                           arrowcolor='rgb(0, 0, 255)')

        if (veh.striking):
            # normal impact plane
            fig.add_annotation(x=veh.impact_norm_X,
                               y=veh.impact_norm_Y,
                               axref='x',
                               ayref='y',
                               text="",
                               showarrow=True,
                               ax=veh.pimpact_X,
                               ay=veh.pimpact_Y,
                               arrowsize=2,
                               arrowhead=2,
                               arrowwidth=1.5,
                               arrowcolor='rgb(153, 0, 51)')
            # tangential impact plane
            fig.add_annotation(x=veh.impact_tang_X,
                               y=veh.impact_tang_Y,
                               axref='x',
                               ayref='y',
                               text="",
                               showarrow=True,
                               ax=veh.pimpact_X,
                               ay=veh.pimpact_Y,
                               arrowsize=1,
                               arrowhead=1,
                               arrowwidth=1,
                               arrowcolor='rgb(153, 0, 51)')

            fig.add_trace(go.Scatter(x=[veh.pimpact_X, veh.pimpact_X], y=[veh.pimpact_Y, veh.pimpact_Y],
                                     mode='markers',
                                     name=f'POI - {veh.name}',
                                     marker=dict(color='rgb(153, 0, 51)', size=7),
                                     ))

    fig.update_layout(
        showlegend=False,
        autosize=False,
        width=width,
        height=width / aspect_ratio,
        title='Initial Vehicle Positions',
        template='plotly_white',
        xaxis=dict(showgrid=True, title='Forward (ft)'),
        yaxis=dict(showgrid=True, title='Rightward (ft)'),
        font=dict(family='Arial', size=22, color='black'))

    fig.update_xaxes(showline=True, linewidth=1, linecolor='black', ticks="outside", range=[-20, 20],
                     tickwidth=1, tickcolor='black', ticklen=10, zeroline=False)
    fig.update_yaxes(autorange="reversed", showline=True, linewidth=1, linecolor='black', ticks="outside", range=[-20 * aspect_ratio, 20 * aspect_ratio],
                     tickwidth=1, tickcolor='black', ticklen=10, zeroline=False)

    fig.show()

    # Table of initial conditions


"""
    fig = go.Figure(data=[go.Table(
    header=dict(values=['Vehicle', 'Vx (mph)', 'Vy (mph)', 'V (mph)', 'Omega (deg/s)', 'X Position (ft)', 'Y Position (ft)'],
                line_color='darkslategray',
                fill_color='lightskyblue',
                align='left'),
    cells=dict(values=[[f'{vehicle_list[0].name}', f'{vehicle_list[1].name}'],
                       [vehicle_list[0].vx_initial, vehicle_list[1].vx_initial],
                       [vehicle_list[0].vy_initial, vehicle_list[1].vy_initial],
                       [np.sqrt(vehicle_list[0].vx_initial**2 + vehicle_list[0].vy_initial**2), np.sqrt(vehicle_list[1].vx_initial**2 + vehicle_list[1].vy_initial**2)],
                       [vehicle_list[0].omega_z, vehicle_list[1].omega_z],
                       [vehicle_list[0].init_x_pos, vehicle_list[1].init_x_pos],
                       [vehicle_list[0].init_y_pos, vehicle_list[1].init_y_pos]],
               line_color='darkslategray',
               fill_color='lightcyan',
               align='left'))
               ])

    fig.update_layout(width=500, height=300)
    fig.show()
"""



"""
create a collision plane on the striking vehicle
this will indicate the contact point and direction of the nornmal and tangential axis of impact force / momentum balance
"""
from pycrash.visualization.vehicle import plot_impact_points, plot_impact_edge
import math

def define_impact_plane(veh, iplane = True):
    """
    generates a point in vehicle 1 (striking) reference frame
    sideswipe collisions - px, py will determine the extent of vehcle engagement with respect
    to contacting edge in vehicle 2 (struck)
    impact momentum model - px, py will be used along with the impact plane to determine time
    of impact and direction of normal and tangential contact planes
    when plane = True, user will also indicate direction of normal impact plane
    """
    user_loc = False
    # set impact edge to zero
    veh.striking = True

    # test for required inputs
    if not veh.lcgf:
        veh.lcgf = float(input("Enter CG to front axle (ft)"))

    if not veh.lcgr:
        veh.lcgr = float(input("Enter CG to rear axle (ft)"))

    if not veh.f_hang:
        veh.f_hang = float(input("Enter front overhang (ft)"))

    if not veh.r_hang:
        veh.r_hang = float(input("Enter rear overhang (ft)"))

    if not veh.width:
        veh.width = float(input("Enter vehicle width (ft)"))

    # create figure of vehicle 1 with scale / grid and p1, p2, p3, p4 labeled when function is called
    # option 5 = custom location
    plot_impact_points(veh, iplane = False) # plot vehicle points

    impact_option = int(input("Choose option for impact point (1, 2, 3, 4, custom = 99: "))

    if impact_option not in [1, 2, 3, 4, 99]:
        print("Invalid impact point option - enter 1, 2, 3, 4 or 5")
        impact_option = int(input("Choose option for impact location"))
    elif impact_option != 99:
        if impact_option == 1:
            veh.pimpact_x = veh.lcgf + veh.f_hang
            veh.pimpact_y = -1 * veh.width / 2
        elif impact_option == 2:
            veh.pimpact_x = veh.lcgf + veh.f_hang
            veh.pimpact_y = veh.width / 2
        elif impact_option == 3:
            veh.pimpact_x = -1 * veh.lcgr - veh.r_hang
            veh.pimpact_y = veh.width / 2
        elif impact_option == 4:
            veh.pimpact_x = -1 * veh.lcgr - veh.r_hang
            veh.pimpact_y = -1* veh.width / 2
    elif impact_option == 99:
        veh.pimpact_x = float(input("Enter x-coordinate ( + forward) of impact point in vehicle frame (ft):"))
        veh.pimpact_y = float(input("Enter y-coordinate ( + rightward) of impact point in vehicle frame (ft):"))
        user_loc = True

    if iplane:
        veh.impact_norm_rad = float(input("Enter heading angle of normal impact plane:  ")) * math.pi / 180
        norm_length = veh.width
        tang_length = veh.width / 2

        veh.impact_norm_x = veh.pimpact_x + norm_length * math.cos(veh.impact_norm_rad)
        veh.impact_norm_y = veh.pimpact_y + norm_length * math.sin(veh.impact_norm_rad)

        veh.impact_tang_x = veh.pimpact_x + tang_length * math.cos(veh.impact_norm_rad + math.pi / 2)
        veh.impact_tang_y = veh.pimpact_y + tang_length * math.sin(veh.impact_norm_rad + math.pi / 2)

    plot_impact_points(veh, user_loc) # plot vehicle points

    return veh

def define_impact_edge(veh, iplane = False):
    """
    generates an edge in vehicle 2 (struck) reference frame
    sideswipe collisions - edge will determine the extent of vehcle engagement with respect
    to contacting point in vehicle 1 (striking)
    impact momentum model - impact edge will be used along with the impact plane to determine time
    of impact
    """
    veh.striking = False

    # test for required inputs
    if not veh.lcgf:
        veh.lcgf = float(input("Enter CG to front axle (ft)"))

    if not veh.lcgr:
        veh.lcgr = float(input("Enter CG to rear axle (ft)"))

    if not veh.f_hang:
        veh.f_hang = float(input("Enter front overhang (ft)"))

    if not veh.r_hang:
        veh.r_hang = float(input("Enter rear overhang (ft)"))

    if not veh.width:
        veh.width = float(input("Enter vehicle width (ft)"))

    # create figure of vehicle 1 with scale / grid and p1, p2, p3, p4 labeled when function is called
    # option 5 = custom location

    plot_impact_edge(veh)

    impact_option = int(input("Choose option for impact edge: "))

    if impact_option not in [1, 2, 3, 4]:
        raise ValueError("Invalid impact edge option - enter 1, 2, 3, 4")
    else:
        if impact_option == 1:
            veh.edgeimpact = 1
            veh.edgeimpact_x1 = veh.lcgf + veh.f_hang
            veh.edgeimpact_y1 = -1 * veh.width / 2
            veh.edgeimpact_x2 = veh.lcgf + veh.f_hang
            veh.edgeimpact_y2 = veh.width / 2
        elif impact_option == 2:
            veh.edgeimpact = 2
            veh.edgeimpact_x1 = veh.lcgf + veh.f_hang
            veh.edgeimpact_y1 = veh.width / 2
            veh.edgeimpact_x2 = -1 * veh.lcgr - veh.r_hang
            veh.edgeimpact_y2 = veh.width / 2
        elif impact_option == 3:
            veh.edgeimpact = 3
            veh.edgeimpact_x1 = -1 * veh.lcgr - veh.r_hang
            veh.edgeimpact_y1 = veh.width / 2
            veh.edgeimpact_x2 = -1 * veh.lcgr - veh.r_hang
            veh.edgeimpact_y2 = -1* veh.width / 2
        elif impact_option == 4:
            veh.edgeimpact = 4
            veh.edgeimpact_x1 = -1 * veh.lcgr - veh.r_hang
            veh.edgeimpact_y1 = -1 * veh.width / 2
            veh.edgeimpact_x2 = veh.lcgf + veh.f_hang
            veh.edgeimpact_y2 = -1 * veh.width / 2

    return veh



import dash
import dash_core_components as dcc
import dash_html_components as html
import plotly.express as px
import pandas as pd

# %%
external_stylesheets = ['https://codepen.io/chriddyp/pen/bWLwgP.css']

app = dash.Dash(__name__, external_stylesheets=external_stylesheets)
# assume you have a "long-form" data frame
# see https://plotly.com/python/px-arguments/ for more options
df = pd.DataFrame({
    "Fruit": ["Apples", "Oranges", "Bananas", "Apples", "Oranges", "Bananas"],
    "Amount": [4, 1, 2, 2, 4, 5],
    "City": ["SF", "SF", "SF", "Montreal", "Montreal", "Montreal"]
})

fig = px.bar(df, x="Fruit", y="Amount", color="City", barmode="group")

app.layout = html.Div(children=[
    html.H1(children='Hello World'),

    html.Div(children='''
        Dash: A web application framework for Python.
    '''),

    dcc.Graph(
        id='example-graph',
        figure=fig
    )
])

if __name__ == '__main__':
    app.run_server(debug=True)




import pandas as pd
import numpy as np
import csv
import os
dt_impact = 0.001
mu_max = 0.9

"""
Performs interative calculations for the SDOF impact simulation
"""
# TODO: update to take user defined inputs

# Functions
# v2 brake force wil oppose the spring force as long as the vehicle is moving forward
def BrakeCheck(brake_applied, springF, v):
    if v > 0:                 # vehicle is in motion - full brake force applied
        return brake_applied * -1 * sign(v)
    if brake_applied == 0:    # no brake applied - brake force will be zero
        return 0
    elif abs(springF) > brake_applied: # spring force is greater than brake force full brake force applied
        return brake_applied * -1
    elif abs(springF) < brake_applied: # spring force is less than braking force so accel will = 0
            return springF * -1

def sign(x):
    # returns the sign of a number
    if x > 0:
        return 1
    elif x < 0:
        return -1
    elif x == 0:
        return 0
    else:
        return x

def SingleDOFmodel(W1, v1_initial, v1_brake, W2, v2_initial, v2_brake, k, cor,
                    tstop, ktype, ttype):

    _tstop = 0 # default limit for ttype == 0 will use tstop otherwise

    if (ttype == 0):
        tstop = _tstop

    # import correct spring force model based on model type
    if (ktype == 'constantK'):
        from .spring_models import SpringForce as SpringForce  # TODO: fix import name
        input_k_disp = 0                # unused input so same function can be used for both conditions
        input_k_force = 0
    elif (ktype == 'tableK'):
        from .spring_models import SpringFdx as SpringForce
        # inputs for model
        input_k_disp = k.iloc[:, 0]
        input_k_force = k.iloc[:, 1]
        k = 0                           # unused input for table based stiffness
    else:
        print("Unknown Model Type")

    # initialize data columns
    columns = ['t', 'x1', 'x2', 'v1', 'v2', 'a1', 'a2', 'springF', 'dx', 'v1_brakeF', 'v2_brakeF']
    spring_model = pd.DataFrame(columns = columns)

    ##################################### SDOF Model ############################################
    # model will produce increasing mutual crush until a common velocity is reached, then mutual crush will decrease
    # model will then use COR to determine the slope back to residual crush
    # model assumes vehicles are in contact at t = 0, dx = 0
    # assumes equal weight distribution on all four tires

    i = 0

    # main loop will run when vehicles are engaged (spring is compressed)
    closing = 1       #  1 if vehicles are closing together, 0 after common velocity is met
    kreturn = np.nan  # initial values of nan
    dxperm = np.nan   # initial values of nan
    stop = 0
    print("")
    print('Model Initiated ============>')
    while stop == 0:
        if v2_initial >= v1_initial:
            print('Vehicles are seperating at onset: Stopping Model')
            stop = 1
            break

        t = i * dt_impact

        if i == 0:

            # initial positions
            x1 = 0
            x2 = 0
            dx = x2 - x1

            # Initial Velocity - convert mph to fps
            v1 = v1_initial * 1.46667
            v2 = v2_initial * 1.46667

            # Initial Brake Applied - max available braking force
            v1_brakeFApp = v1_brake * W1 * mu_max
            v2_brakeFApp = v2_brake * W2 * mu_max

            #print(f'Brake Force Applied by V1 = {v1_brakeFApp} lb')
            #print(f'Brake Force Applied by V2 = {v2_brakeFApp} lb')

            # Initial Brake Force opposing vehicle velocity
            v1_brakeF = v1_brakeFApp * -1 * sign(v1)   # Initial brake force
            v2_brakeF = v2_brakeFApp * -1 * sign(v2)   # Initial brake force

            # Initial Acceleration - only due to braking
            a1 = v1_brakeF * 32.2 / W1
            a2 = v2_brakeF * 32.2 / W2

            springF = 0  # no initial spring force

            # store initial data in dataframe
            data = [t, x1, x2, v1, v2, a1, a2, springF, dx, v1_brakeF, v2_brakeF]
            spring_model = spring_model.append(pd.Series(data, index = columns), ignore_index=True)


        # this section will calculate forces based on closing or seperating phase
        if i > 0:

            # calculate vehicle motion [ft] based on prior velocity
            x1 = x1 + v1 * dt_impact
            x2 = x2 + v2 * dt_impact

            if closing == 1:
            # crush is equal to difference in vehicle motion during closing
                dx = x2 - x1
            elif (closing == 0) & ((x2-x1) < 1*dxperm):
            # crush is equal to the difference in vehicle motion until dxperm is reached
                dx = x2 - x1
            elif (closing == 0) & ((x2-x1) >= 1*dxperm):
            # crush remains at dxperm once vehicles seperate beyond dxperm
            # force should be zero at this point
                dx = dxperm

            # calculate vehicle velocity [ft/s] based on prior acceleration
            v1 = v1 + a1 * dt_impact
            v2 = v2 + a2 * dt_impact

            dx_past = spring_model.loc[i-1, 'dx']
            # check for closing / seperating status
            if closing == 1:
                if abs(dx_past) > abs(dx):  # if mutual crush decreased at current time step, vehicles are seperating
                    closing = 0
                    Fmax = spring_model.loc[i-1, 'springF']
                    dxmax = dx_past
                    dxperm = dxmax * (1 - cor**2)  # permanent crush
                    kreturn = Fmax / abs(dxmax - dxperm)  # return stiffness
                    print('===== Seperation ===========>')
                    print(f'Time (s) = {t}')
                    print(f'Peak Mutual Crush (in) = {dx_past *-12:.2f}')
                    print(f'Peak Force (lb) = {Fmax:.2f}')

            springF = SpringForce(dx, closing, k, input_k_disp, input_k_force, kreturn, dxperm)

            # calculate brake force based on applied braking and spring force
            # v1 brake force will always oppose v1 velocity
            v1_brakeF = v1_brakeFApp * -1 * sign(v1)
            v2_brakeF = BrakeCheck(v2_brakeFApp, springF, v2) # applied brake will oppose velocity if vehicle is in motion

            # calculate vehicle accleration [ft/s/s]
            a1 = 32.2 / W1 * (v1_brakeF - springF)
            a2 = 32.2 / W2 * (v2_brakeF + springF)

            if (closing == 0) & (dx >= dxperm) & (t >= tstop): # vehicles are seperating and dx is less than permanent crush
                stop = 1
                print("")
                print(f'========> Stopped t (s) = {t:.3f}')
                break

            # store initial data in dataframe
            data = [t, x1, x2, v1, v2, a1, a2, springF, dx, v1_brakeF, v2_brakeF]
            spring_model = spring_model.append(pd.Series(data, index = columns), ignore_index=True)

        i += 1

    return spring_model



"""
function for processing load cell barrier data from NHTSA
load cell data are downloaded individually as .txt files
displacement data is downloaded as a single .txt file
functions below will merge force and displacement data into a single dataframe for analyses
"""

# %% modules
import os
path_parent = os.path.dirname(os.getcwd())
data_directory = os.path.join(path_parent, "project", "demo", "input")
os.chdir(path_parent)

import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
from src.project import Project, project_info, load_project
from src.vehicle import Vehicle
from src.sdof_model import SDOF_Model
from src.functions import cipriani

# %% Test Data info
test_info = {'name':'NHTSA 6953', 'number':6953, 'vehicle':'2011 Toyota Camry'}

# %%  get list of files contained in directory with NHTSA load cell data
os.chdir('D:\\OneDrive\\BIOCORE Cases\\Gerard\\JMC Work\\veh2\\nhtsa6953')
os.listdir()


# %%  copy output from above and past below
# channel names
file_names = ['v06953_106fa0_B1.txt',
 'v06953_107fa0_B2.txt',
 'v06953_108fa0_B3.txt',
 'v06953_109fa0_B4.txt',
 'v06953_110fa0_B5.txt',
 'v06953_111fa0_B6.txt',
 'v06953_112fa0_B7.txt',
 'v06953_113fa0_B8.txt',
 'v06953_114fa0_B9.txt',
 'v06953_115fa0_C1.txt',
 'v06953_116fa0_C2.txt',
 'v06953_117fa0_C3.txt',
 'v06953_118fa0_C4.txt',
 'v06953_119fa0_C5.txt',
 'v06953_120fa0_C6.txt',
 'v06953_121fa0_C7.txt',
 'v06953_122fa0_C8.txt',
 'v06953_123fa0_C9.txt',
 'v069AVG_disp.txt']

# create list of user friendly variable names
chnames = ['B1', 'B2', 'B3', 'B4', 'B5', 'B6', 'B7', 'B8', 'B9',
           'C1', 'C2', 'C3', 'C4', 'C5', 'C6', 'C7', 'C8', 'C9', 'disp']

# %% Load NHTSA Data and combine into a single dataframe
for i in range(len(file_names)):
        channel = chnames[i]
        SingleChannel = pd.read_csv(file_names[i], sep = '\t', names = ['time', channel])
        if i == 0:
            df = SingleChannel
        if i > 0:
            df = pd.merge(df, SingleChannel, how = 'left', on = 'time')        

        del SingleChannel

print(f'Combined Load Cell data of shape = {df.shape}')
print('Data is in SI units [N], [mm]')
df.head()

# %% Combined data by columns - 
df['lower'] = df[['B1', 'B2', 'B3', 'B4', 'B5', 'B6', 'B7', 'B8', 'B9']].sum(axis = 1)
df['upper'] = df[['C1', 'C2', 'C3', 'C4', 'C5', 'C6', 'C7', 'C8', 'C9']].sum(axis = 1)
df['total'] = df[['lower', 'upper']].sum(axis = 1)


# %% Plot Functions - English
def load_cell_fdt(df, t_max = 0.150):
    plt.figure(figsize = (19,10))
    plt.title(f"NHTSA {test_info['number']} - {test_info['vehicle']} - Load Cell Barrier Forces", fontsize=22)
    plt.ylabel('Force (lb)', fontsize=16)
    plt.xlabel('Time (s)', fontsize=16)
    plt.xticks(fontsize=16)
    plt.yticks(fontsize=16)
    plt.gca().spines['top'].set_visible(False)
    plt.gca().spines['right'].set_visible(False)
    plt.plot(df.time, df.lower * 0.224809, linewidth = 2, marker = None, c = 'green')
    plt.plot(df.time, df.upper * 0.224809, linewidth = 2, marker = None, c = 'blue')
    plt.plot(df.time, df.total * 0.224809, linewidth = 2, marker = None, c = 'black')
    plt.legend(['Bumper - Lower Row', 'Bumper - Upper Row', 'Bumper - Total'], frameon=False, prop={'size': 14}, loc = 4)
    plt.xlim(0, t_max)
    plt.ylim(min(df.total) * 0.224809 - 5000, max(df.total)* 0.224809 + 5000)
    return plt.show()

def load_cell_fdx(df):
    plt.figure(figsize = (19,10))
    plt.title(f"NHTSA {test_info['number']} - {test_info['vehicle']} - Load Cell Barrier Forces", fontsize=22)
    plt.ylabel('Force (lb)', fontsize=16)
    plt.xlabel('Displacement (in)', fontsize=16)
    plt.xticks(fontsize=16)
    plt.yticks(fontsize=16)
    plt.gca().spines['top'].set_visible(False)
    plt.gca().spines['right'].set_visible(False)
    plt.plot(df.disp * 0.0393701, df.lower * 0.224809, linewidth = 2, marker = None, c = 'green')
    plt.plot(df.disp * 0.0393701, df.upper * 0.224809, linewidth = 2, marker = None, c = 'blue')
    plt.plot(df.disp * 0.0393701, df.total * 0.224809, linewidth = 2, marker = None, c = 'black')
    plt.legend(['Bumper - Lower Row', 'Bumper - Upper Row', 'Bumper - Total'], frameon=False, prop={'size': 14}, loc = 3)
    plt.xlim(0, max(df.disp) * 0.0393701 + 5)
    plt.ylim(min(df.total) * 0.224809 - 5000, max(df.total)* 0.224809 + 5000)

# %% call plot functions
load_cell_fdt(df)
load_cell_fdx(df)

# %% case specific - max dispacements of interest
# create new dataframe limited to <= 0.1 seconds to eliminate rebound
df_model = df[df.time <= 0.1].copy()
#convert displacement to feet and force to lb
df_model['disp_ft'] = df_model.disp.apply(lambda x: x * 0.00328084)
df_model['total_lb'] = df_model.total.apply(lambda x: x * 0.2248090795)
df_model['upper_lb'] = df_model.upper.apply(lambda x: x * 0.2248090795)
df_model['lower_lb'] = df_model.lower.apply(lambda x: x * 0.2248090795)

# use lower bumper force up to 3.3 inches, then only use upper
df_model['model_force'] = np.where((df_model.disp_ft < (3.3/12)), df_model.total_lb, df_model.upper_lb)

# %% plot columns of interest

def plot_fdx(x, y, plot_title = False):
    plt.figure(figsize = (19,10))
    if (plot_title):
        plt.title(plot_title, fontsize=22)
    
    plt.ylabel('Force (lb)', fontsize=16)
    plt.xlabel('Displacement (in)', fontsize=16)
    plt.xticks(fontsize=16)
    plt.yticks(fontsize=16)
    plt.gca().spines['top'].set_visible(False)
    plt.gca().spines['right'].set_visible(False)
   # plt.plot(x * 0.0393701, y * 0.224809, linewidth = 2, marker = None, c = 'green')
   # plt.plot(x * 0.0393701, y * 0.224809, linewidth = 2, marker = None, c = 'blue')
    plt.plot(x * 12, y, linewidth = 2, marker = None, c = 'black')
   # plt.legend(['Bumper - Lower Row', 'Bumper - Upper Row', 'Bumper - Total'], frameon=False, prop={'size': 14}, loc = 3)
    plt.xlim(0, max(x) * 12 + 5)
    plt.ylim(min(y) - 1000, max(y) + 1000)

plot_title = f"NHTSA {test_info['number']} - {test_info['vehicle']} - Load Cell Barrier Forces"
plot_fdx(df_model.disp_ft, df_model.model_force, plot_title)

# %% get percentage of force applied to vehicle by ramp
impactor_height = 0.0254 # [m]
force_height = 2 * 0.246 # [m]
trailer_ratio = impactor_height / force_height

plot_title = f"Trailer Impact Force: Total / Upper Split"
df_model['trailer_force'] = df_model.model_force.apply(lambda x: x * trailer_ratio)
plot_fdx(df_model.disp_ft, df_model.trailer_force, plot_title)

# %%
df_model['total_force_trailer'] = df_model.total_lb.apply(lambda x: x * trailer_ratio)
plot_title = f"Trailer Impact Force: Total"
plot_fdx(df_model.disp_ft, df_model.total_force_trailer, plot_title)


# %% Create Project instance
project_inputs = {'name':'Gerard', 'pdesc':'low speed frontal', 'sim_type':'MV', 'impact_type':'SDOF',
                  'note':'backing trailer with ramp'}

proj = Project(project_inputs)

# %% Create Vehicle Instances


vehicle_input_dict = {"year":2007,
"make":"Chevy",
"model":"Sierra C250",
"weight":3200,
"vin":"1GTHC24KX7E564611",
"brake":0,
"steer_ratio":16.5,
"init_x_pos":0,
"init_y_pos":0,
"head_angle":0,
"v_width":6,
"v_length":19.3,
"hcg":2,
"lcgf":4.88,
"lcgr":6.96,
"wb":11.84,
"track":6.6,
"f_hang":3.2,
"r_hang":4.1,
"tire_d":2.716666667,
"tire_w":0.866666667,
"izz":2500,
"fwd":0,
"rwd":1,
"awd":0,
"A":100,
"B":41,
"k":1000,
"L":0,
"c":0,
"vx_initial":10,
"vy_initial":0,
"omega_z":0,
"driver_input":driver_input_df}

veh1 = Vehicle('Veh1', vehicle_input_dict)
# %% Build model stiffness that will use load cell data from above
kmodel = df_model[['disp_ft', 'total_force_trailer']].copy()

v1_initial = [3, 4, 5]            # initial speeds for striking vehicle
cor_list = [cipriani(i) for i in v1_initial]
colorList = ['k', 'b', 'g']
run_list = ['run1', 'run2', 'run3']
for i in range(len(v1_initial)):
    print(i)
    model_inputs = {"name":run_list[i],
                    "k":kmodel,
                    "cor":cor_list[i], 
                    "tstop":None
                }

    veh1.vx_initial = v1_initial[i]
    run_list[i] = SDOF_Model(veh1, veh2, model_inputs)


# %%



import pandas as pd
import numpy as np
from copy import deepcopy
from .vehicle_model import vehicle_model
from pycrash.model_calcs.position_data import position_data_motion
from .visualization.vehicle import plot_driver_inputs
from .visualization.kinematics import plot_model
from .visualization.model import plot_motion
from .visualization.tire_details import tire_details, vertical_forces
import os

# TODO: create inputs for plots
figure_size = (16, 9)

# look for Environment data, load if present
if os.path.isfile(os.path.join(os.getcwd(), "data", "input", "environment.csv")):
    enviro = pd.read_csv(os.path.join(os.getcwd(), "data", "input", "environment.csv"))
    if len(enviro) == 0:
        print('Environment file appears blank - no terrian data used')
        print(f'Constant friction {mu_max} used throughout')
    else:
        print('TODO - process terrain data')
else:
    print('No Environment File Provided')


class SingleMotion:
    """
    Generates vehicle motion based on inputs defined within Vehicle class
    No external forces
    Environment slope and bank is optionally defined as a function of X,Y components
    creates independent copy of vehicle at instantiation
    """

    def __init__(self, name, veh, user_sim_defaults=None):
        """
        default values necessary for single motion simulation are loaded when a vehicle is instantiated
        """
        # load defaults
        if user_sim_defaults:
            # TODO: create check for user sim_defaults_input
            sim_defaults = user_sim_defaults
        else:
            sim_defaults = {'dt_motion': 0.01,
                            'mu_max': 0.76,
                            'alpha_max': 0.174533}

        self.mu_max = sim_defaults['mu_max']  # maximum available friction
        self.dt_motion = sim_defaults['dt_motion']  # iteration time step
        self.alpha_max = sim_defaults['alpha_max']  # maximum tire slip angle (rad)

        print(f"Maximum allowable friction: {self.mu_max}")
        print(f"Time step for vehicle motion (s) : {self.dt_motion}")
        print(f"Maximum tire slip angle (deg): {self.alpha_max * 180 / 3.14159:0.2f}")

        self.name = name
        self.type = 'singlemotion'  # class type for saving files
        self.veh = deepcopy(veh)
        # check for driver inputs
        if isinstance(self.veh.driver_input, pd.DataFrame):
            print(f"Driver input for {self.veh.name} of shape = {self.veh.driver_input.shape}")
        else:
            print(f'Driver input for {self.veh.name} not provided - no braking or steering applied')
            print(f'Current driver input of type: {type(self.veh.driver_input)}')
            end_time = int(input('Enter duration for simulation (seconds):'))
            t = list(np.arange(0, end_time + dt_motion, dt_motion))  # create time array from 0 to end time from user
            throttle = [0] * len(t)
            brake = [0] * len(t)
            steer = [0] * len(t)
            driver_input_dict = {'t': t, 'throttle': throttle, 'brake': brake, 'steer': steer}
            self.veh.driver_input = pd.DataFrame.from_dict(driver_input_dict)
            print(f'Driver inputs for {self.veh.name} set to zero for {end_time} seconds')

        # TODO: need to check if time in driver_inputs is the correct dt?
        # TODO: check for init_x_pos, init_y_pos

        # run planar vehicle motion simulation
        self.veh = vehicle_model(self.veh, sim_defaults)

        # create point data in vehicle and global frame
        self.veh = position_data_motion(self.veh)

    def plot_inputs(self):
        plot_driver_inputs(self.veh)

    def vehicle_info(self):
        """
        get input on the vehicle used to create Kinematics object
        """
        print(f'Vehicle name is {self.veh.name}')

    def plot_model(self):
        plot_model(self.veh)

    def global_motion(self, i):
        plot_motion(self.veh, i)

    def tire_detail(self):
        tire_details(self.veh)
        vertical_forces(self.veh)

    def CG_motion(self):
        """
        plot location of CG in global reference frame
        """
        fig = plt.figure(figsize=figure_size)
        ax = fig.gca()
        # determine extent of vehicle plot
        min_x_axis = min(self.veh.model.Dx) - 20
        max_x_axis = max(self.veh.model.Dx) + 20

        min_y_axis = min(self.veh.model.Dy) - 20
        max_y_axis = max(self.veh.model.Dy) + 20

        ax.set_xticks(np.arange(min_x_axis, max_x_axis, 20))
        ax.set_yticks(np.arange(min_y_axis, max_y_axis, 20))
        plt.scatter(self.veh.model.Dx, self.veh.model.Dy, label=f'{self.veh.name}')

        plt.xlim([min_x_axis, max_x_axis])
        plt.ylim([min_y_axis, max_y_axis])

        plt.grid()
        plt.gca().invert_yaxis()
        plt.show()

    def draw_vehicle(self, i):
        """
        plots vehicle motion along with velocity vector and coordinate axes
        """

        #  These dataframes link the components together for plotting
        bdy_x = (self.p_vx.b_lfc[i], self.p_vx.b_rfc[i], self.p_vx.b_rrc[i], self.p_vx.b_lrc[i], self.p_vx.b_lfc[i])
        bdy_y = (self.p_vy.b_lfc[i], self.p_vy.b_rfc[i], self.p_vy.b_rrc[i], self.p_vy.b_lrc[i], self.p_vy.b_lfc[i])

        lfw_x = (self.p_vx.lfw_a[i], self.p_vx.lfw_b[i], self.p_vx.lfw_c[i], self.p_vx.lfw_d[i], self.p_vx.lfw_a[i])
        lfw_y = (self.p_vy.lfw_a[i], self.p_vy.lfw_b[i], self.p_vy.lfw_c[i], self.p_vy.lfw_d[i], self.p_vy.lfw_a[i])

        rfw_x = (self.p_vx.rfw_a[i], self.p_vx.rfw_b[i], self.p_vx.rfw_c[i], self.p_vx.rfw_d[i], self.p_vx.rfw_a[i])
        rfw_y = (self.p_vy.rfw_a[i], self.p_vy.rfw_b[i], self.p_vy.rfw_c[i], self.p_vy.rfw_d[i], self.p_vy.rfw_a[i])

        rrw_x = (self.p_vx.rrw_a[i], self.p_vx.rrw_b[i], self.p_vx.rrw_c[i], self.p_vx.rrw_d[i], self.p_vx.rrw_a[i])
        rrw_y = (self.p_vy.rrw_a[i], self.p_vy.rrw_b[i], self.p_vy.rrw_c[i], self.p_vy.rrw_d[i], self.p_vy.rrw_a[i])

        lrw_x = (self.p_vx.lrw_a[i], self.p_vx.lrw_b[i], self.p_vx.lrw_c[i], self.p_vx.lrw_d[i], self.p_vx.lrw_a[i])
        lrw_y = (self.p_vy.lrw_a[i], self.p_vy.lrw_b[i], self.p_vy.lrw_c[i], self.p_vy.lrw_d[i], self.p_vy.lrw_a[i])

        # Plot Vehicle in Vehicle reference frame
        plt.figure(figsize=figure_size)
        plt.xlim([-20, 20])
        plt.ylim([-10, 10])

        plt.plot(bdy_x, bdy_y, 'k')
        plt.scatter(self.p_vx.lfw[i], self.p_vy.lfw[i], c='b')  # left front wheel center
        plt.plot(lfw_x, lfw_y, 'b')
        plt.scatter(self.p_vx.rfw[i], self.p_vy.rfw[i], c='g')  # right front wheel center
        plt.plot(rfw_x, rfw_y, 'g')
        plt.scatter(self.p_vx.rrw[i], self.p_vy.rrw[i], c='m')  # right rear wheel center
        plt.plot(rrw_x, rrw_y, 'm')
        plt.scatter(self.p_vx.lrw[i], self.p_vy.lrw[i], c='orange')  # left rear wheel center
        plt.plot(lrw_x, lrw_y, 'orange')

        # vehicle CG
        plt.scatter(self.p_vx.cg[i], self.p_vy.cg[i], s=500, c='k')

        # velocity vector
        plt.arrow(self.p_vx.cg[i], self.p_vy.cg[i], self.p_vx.vel_v[i] - self.p_vx.cg[i],
                  self.p_vy.vel_v[i] - self.p_vy.cg[i], head_width=.5, head_length=0.5, fc='r', ec='r')

        # vehicle axes
        plt.arrow(self.p_vx.cg[i], self.p_vy.cg[i], self.p_vx.xaxis[i] - self.p_vx.cg[i],
                  self.p_vy.xaxis[i] - self.p_vy.cg[i], head_width=.5, head_length=0.5, fc='k', ec='k')
        plt.arrow(self.p_vx.cg[i], self.p_vy.cg[i], self.p_vx.yaxis[i] - self.p_vx.cg[i],
                  self.p_vy.yaxis[i] - self.p_vy.cg[i], head_width=.5, head_length=0.5, fc='b', ec='b')
        plt.gca().invert_yaxis()
        plt.show()

    def draw_vehicle_motion(self, i, tire_path=True):
        """
        Plot Vehicle in Global reference frame
        """

        bdy_x = (self.p_gx.b_lfc[i], self.p_gx.b_rfc[i], self.p_gx.b_rrc[i], self.p_gx.b_lrc[i], self.p_gx.b_lfc[i])
        bdy_y = (self.p_gy.b_lfc[i], self.p_gy.b_rfc[i], self.p_gy.b_rrc[i], self.p_gy.b_lrc[i], self.p_gy.b_lfc[i])

        lfw_x = (self.p_gx.lfw_a[i], self.p_gx.lfw_b[i], self.p_gx.lfw_c[i], self.p_gx.lfw_d[i], self.p_gx.lfw_a[i])
        lfw_y = (self.p_gy.lfw_a[i], self.p_gy.lfw_b[i], self.p_gy.lfw_c[i], self.p_gy.lfw_d[i], self.p_gy.lfw_a[i])

        rfw_x = (self.p_gx.rfw_a[i], self.p_gx.rfw_b[i], self.p_gx.rfw_c[i], self.p_gx.rfw_d[i], self.p_gx.rfw_a[i])
        rfw_y = (self.p_gy.rfw_a[i], self.p_gy.rfw_b[i], self.p_gy.rfw_c[i], self.p_gy.rfw_d[i], self.p_gy.rfw_a[i])

        rrw_x = (self.p_gx.rrw_a[i], self.p_gx.rrw_b[i], self.p_gx.rrw_c[i], self.p_gx.rrw_d[i], self.p_gx.rrw_a[i])
        rrw_y = (self.p_gy.rrw_a[i], self.p_gy.rrw_b[i], self.p_gy.rrw_c[i], self.p_gy.rrw_d[i], self.p_gy.rrw_a[i])

        lrw_x = (self.p_gx.lrw_a[i], self.p_gx.lrw_b[i], self.p_gx.lrw_c[i], self.p_gx.lrw_d[i], self.p_gx.lrw_a[i])
        lrw_y = (self.p_gy.lrw_a[i], self.p_gy.lrw_b[i], self.p_gy.lrw_c[i], self.p_gy.lrw_d[i], self.p_gy.lrw_a[i])

        fig = plt.figure(figsize=figure_size)
        # plt.xlim([-300, 50])
        # plt.ylim([-100, 100])

        plt.plot(bdy_x, bdy_y, 'k')
        plt.scatter(self.p_gx.lfw[i], self.p_gy.lfw[i], c='b')  # left front wheel center
        plt.plot(lfw_x, lfw_y, 'b')
        plt.scatter(self.p_gx.rfw[i], self.p_gy.rfw[i], c='g')  # right front wheel center
        plt.plot(rfw_x, rfw_y, 'g')
        plt.scatter(self.p_gx.rrw[i], self.p_gy.rrw[i], c='m')  # right rear wheel center
        plt.plot(rrw_x, rrw_y, 'm')
        plt.scatter(self.p_gx.lrw[i], self.p_gy.lrw[i], c='orange')  # left rear wheel center
        plt.plot(lrw_x, lrw_y, 'orange')
        plt.scatter(self.p_gx.cg[i], self.p_gy.cg[i], s=100, c='k')  # vehicle CG
        # velocity vector
        plt.arrow(self.p_gx.cg[i], self.p_gy.cg[i], self.p_gx.vel_v[i] * 0.2, self.p_gy.vel_v[i] * 0.2, head_width=.5,
                  head_length=0.5, fc='r', ec='r')
        # vehicle axes
        plt.arrow(self.p_gx.cg[i], self.p_gy.cg[i], self.p_gx.xaxis[i] - self.p_gx.cg[i],
                  self.p_gy.xaxis[i] - self.p_gy.cg[i], head_width=.5, head_length=0.5, fc='k', ec='k')
        plt.arrow(self.p_gx.cg[i], self.p_gy.cg[i], self.p_gx.yaxis[i] - self.p_gx.cg[i],
                  self.p_gy.yaxis[i] - self.p_gy.cg[i], head_width=.5, head_length=0.5, fc='b', ec='b')

        # for loop up to i
        if (tire_path):
            for i in range(0, i):
                if self.p_gx.loc[i, 'lf_lock'] == 0:
                    plt.scatter(self.p_gx.loc[i, 'lfw'], self.p_gy.loc[i, 'lfw'], c='b', s=1, marker='.')
                elif self.p_gx.loc[i, 'lf_lock'] == 1:
                    plt.scatter(self.p_gx.loc[i, 'lfw'], self.p_gy.loc[i, 'lfw'], c='b', s=4, marker='s')

                if self.p_gx.loc[i, 'rf_lock'] == 0:
                    plt.scatter(self.p_gx.loc[i, 'rfw'], self.p_gy.loc[i, 'rfw'], c='g', s=1, marker='.')
                elif self.p_gx.loc[i, 'rf_lock'] == 1:
                    plt.scatter(self.p_gx.loc[i, 'rfw'], self.p_gy.loc[i, 'rfw'], c='g', s=4, marker='s')

                if self.p_gx.loc[i, 'rr_lock'] == 0:
                    plt.scatter(self.p_gx.loc[i, 'rrw'], self.p_gy.loc[i, 'rrw'], c='m', s=1, marker='.')
                elif self.p_gx.loc[i, 'rr_lock'] == 1:
                    plt.scatter(self.p_gx.loc[i, 'rrw'], self.p_gy.loc[i, 'rrw'], c='m', s=4, marker='s')

                if self.p_gx.loc[i, 'lr_lock'] == 0:
                    plt.scatter(self.p_gx.loc[i, 'lrw'], self.p_gy.loc[i, 'lrw'], c='orange', s=1, marker='.')
                elif self.p_gx.loc[i, 'lr_lock'] == 1:
                    plt.scatter(self.p_gx.loc[i, 'lrw'], self.p_gy.loc[i, 'lrw'], c='orange', s=4, marker='s')

        plt.gca().invert_yaxis()
        plt.show()



"""
vehicle motion for mutliple vehicles
"""
from .model_calcs.sideswipe import ss
from .model_calcs.tire_model import tire_forces
from .model_calcs.impact_detect import detect
from .vehicle_model import vehicle_model
from .model_calcs.carpenter_momentum_calcs import impc
import pandas as pd
import numpy as np
from scipy import integrate
import math
import csv
import os

# column list for vehicle model
column_list = ['t', 'vx', 'vy', 'Vx', 'Vy', 'Vr', 'vehicleslip_deg', 'vehicleslip_rad', 'oz_deg', 'oz_rad', 'delta_deg',
               'delta_rad', 'turn_rX', 'turn_rY', 'turn_rR', 'au', 'av', 'ax', 'ay', 'ar', 'Ax', 'Ay', 'Ar',
               'alphaz', 'alphaz_deg', 'beta_deg', 'beta_rad', 'lf_fx', 'lf_fy', 'rf_fx', 'rf_fy',
               'rr_fx', 'rr_fy', 'lr_fx', 'lr_fy', 'lf_alpha', 'rf_alpha', 'rr_alpha', 'lr_alpha',
               'lf_lock', 'rf_lock', 'rr_lock', 'lr_lock', 'lf_fz', 'rf_fz', 'rr_fz', 'lr_fz',
               'theta_rad', 'theta_deg', 'Fx', 'Fy', 'Mz']


# TODO: ignore driver inputs after impact
# TODO: disable tire after impact

def multi_vehicle_model(vehicle_list, sim_defaults, impact_type, ignore_driver=False, kmutual=None, vehicle_mu=None):
    """
    Calculate vehicle dynamics from driver inputs and environmental inputs
    vehicle_list is a list of vehicle class instances [veh1, veh2] - two currently
    model will use the max time from vehicle 1 for input, all vehicles
    need the same total driver input time
    model_type indicates the type of impact to be simulated (sideswipe (ss), or impulse-momentum (impc))
    ignore_driver (False) will use driver_inputs after impact.  True will ignore all driver inputs and keep
    last input for remainder of simulation
    kmutual must be defined by sideswipe simulations
    """
    impact = False
    impc_complete = False  # flag to allow impc model to run only once

    print(f"Impact Type: {impact_type}")
    # load defaults
    dt_motion = sim_defaults['dt_motion']  # iteration time step



    j = 0

    print(f"Two vehicle simulation will run for {max(vehicle_list[0].driver_input.t)} s")

    for veh in vehicle_list:
        veh.model = pd.DataFrame(np.nan, index=np.arange(len(veh.driver_input.t)), columns=column_list)
        veh.model.Fx[0] = 0
        veh.model.Fy[0] = 0
        veh.model.Mz[0] = 0
        veh.model.au[0] = 0  # no initial vehicle pitch
        veh.model.av[0] = 0  # no initial vehicle roll
        veh.model.vx[0] = veh.vx_initial * 1.46667  # convert input in mph to fps
        veh.model.vy[0] = veh.vy_initial * 1.46667  # convert input in mph to fps
        veh.model.theta_rad[0] = veh.head_angle * math.pi / 180  # initial heading angle
        veh.model.oz_rad[0] = veh.omega_z * (math.pi / 180)  # initial angular rate (deg/s) - input
        veh.model.Vx[0] = veh.model.vx[0] * math.cos(veh.model.theta_rad[0]) - veh.model.vy[0] * math.sin(veh.model.theta_rad[0])
        veh.model.Vy[0] = veh.model.vx[0] * math.sin(veh.model.theta_rad[0]) + veh.model.vy[0] * math.cos(veh.model.theta_rad[0])

    for i in (range(len(vehicle_list[0].driver_input.t))):

        # step through each vehicle
        for veh in vehicle_list:
            veh.model.t[i] = round(i * dt_motion, 4)  # assigning time

            # get tire forces for t = 0
            veh = tire_forces(veh, i, sim_defaults)

            # setting vehicle forces to zero if no impact
            # impact may occur as a result of vehicle 2 motion in which case, the forces for t=i will be
            # updated for each vehicle
            if impact_type != 'SS':
                veh.model.Fx[i] = 0
                veh.model.Fy[i] = 0
                veh.model.Mz[i] = 0
            elif (impact_type == 'SS') & (impact == False):
                veh.model.Fx[i] = 0
                veh.model.Fy[i] = 0
                veh.model.Mz[i] = 0

            print(f"Model Fx: {veh.model.Fx[i]} at i: {i}")
            # local vehicle acceleration
            veh.model.au[i] = 32.2 / veh.weight * np.sum([veh.model.lf_fx[i],
                                                          veh.model.rf_fx[i],
                                                          veh.model.rr_fx[i],
                                                          veh.model.lr_fx[i],
                                                          veh.model.Fx[i]])

            veh.model.av[i] = 32.2 / veh.weight * np.sum([veh.model.lf_fy[i],
                                                          veh.model.rf_fy[i],
                                                          veh.model.rr_fy[i],
                                                          veh.model.lr_fy[i],
                                                          veh.model.Fy[i]])

            # rotation acceleration - alpha-z
            veh.model.alphaz[i] = (1 / veh.izz) * np.sum([veh.model.lf_fx[i] * veh.track / 2,
                                                          veh.model.lf_fy[i] * veh.lcgf,
                                                          -1 * veh.model.rf_fx[i] * veh.track / 2,
                                                          veh.model.rf_fy[i] * veh.lcgf,
                                                          -1 * veh.model.rr_fx[i] * veh.track / 2,
                                                          -1 * veh.model.rr_fy[i] * veh.lcgr,
                                                          veh.model.lr_fx[i] * veh.track / 2,
                                                          -1 * veh.model.lr_fy[i] * veh.lcgr,
                                                         veh.model.Mz[i]])

            if i == 0:
                # vehicle acceleration in inertial frame
                veh.model.ax[i] = veh.model.au[i] + veh.model.oz_rad[i] * veh.model.vy[i]
                veh.model.ay[i] = veh.model.av[i] - veh.model.oz_rad[i] * veh.model.vx[i]
                veh.model.ar[i] = math.sqrt(veh.model.ax[i] ** 2 + veh.model.ay[i] ** 2)

                # inertial frame coorindates - capital letters
                veh.model.Ax[i] = veh.model.au[i] * math.cos(veh.model.theta_rad[i]) - veh.model.av[i] * math.sin(veh.model.theta_rad[i])
                veh.model.Ay[i] = veh.model.au[i] * math.sin(veh.model.theta_rad[i]) + veh.model.av[i] * math.cos(veh.model.theta_rad[i])

            if i > 0:
                # vehicle acceleration in inertial frame
                veh.model.ax[i] = veh.model.au[i] + veh.model.oz_rad[i - 1] * veh.model.vy[i - 1]
                veh.model.ay[i] = veh.model.av[i] - veh.model.oz_rad[i - 1] * veh.model.vx[i - 1]
                veh.model.ar[i] = math.sqrt(veh.model.ax[i] ** 2 + veh.model.ay[i] ** 2)

                # vehicle velocities
                veh.model.vx[i] = veh.model.vx[i - 1] + dt_motion * np.mean([veh.model.ax[i - 1], veh.model.ax[i]])
                veh.model.vy[i] = veh.model.vy[i - 1] + dt_motion * np.mean([veh.model.ay[i - 1], veh.model.ay[i]])

                # omega
                veh.model.oz_rad[i] = veh.model.oz_rad[i - 1] + dt_motion * np.mean([veh.model.alphaz[i - 1], veh.model.alphaz[i]])

                # heading angle
                veh.model.theta_rad[i] = veh.model.theta_rad[i - 1] + dt_motion * np.mean([veh.model.oz_rad[i], veh.model.oz_rad[i - 1]])

                # inertial frame coorindates - capital letters
                veh.model.Ax[i] = veh.model.au[i] * math.cos(veh.model.theta_rad[i]) - veh.model.av[i] * math.sin(veh.model.theta_rad[i])
                veh.model.Ay[i] = veh.model.au[i] * math.sin(veh.model.theta_rad[i]) + veh.model.av[i] * math.cos(veh.model.theta_rad[i])

                veh.model.Vx[i] = veh.model.Vx[i - 1] + dt_motion * np.mean([veh.model.Ax[i - 1], veh.model.Ax[i]])
                veh.model.Vy[i] = veh.model.Vy[i - 1] + dt_motion * np.mean([veh.model.Ay[i - 1], veh.model.Ay[i]])

                # velocity vector in inertial frame
                veh.model.beta_rad[i] = math.atan2(veh.model.Vy[i], veh.model.Vx[i])  # move to seperate calc

            # vehicle position
            veh.model['Dx'] = veh.init_x_pos + integrate.cumtrapz(list(veh.model.Vx), list(veh.model.t), initial=0)
            veh.model['Dy'] = veh.init_y_pos + integrate.cumtrapz(list(veh.model.Vy), list(veh.model.t), initial=0)
            veh.model.alphaz_deg = [row * 180 / math.pi for row in veh.model.alphaz]  # move to seperate calc
            veh.model.oz_deg = [row * 180 / math.pi for row in veh.model.oz_rad]  # move to seperate calc
            veh.model.theta_deg = [row * 180 / math.pi for row in veh.model.theta_rad]  # move to seperate calc
            veh.model.beta_deg = [row * 180 / math.pi for row in veh.model.beta_rad]  # move to seperate calc

        # detect impact using current vehicle positions after first iterations
        if i == 0:
            crush_data = None

        if impc_complete == False:
            crush_data = detect(vehicle_list, i, crush_data)  # only neeed to detect impact if it hasn't occured yet in IMPC model

        print(f"Impact Detect: {crush_data.impact[i]} at i: {i}")
        if (crush_data.impact[i] == True) & (impc_complete == False):
            impact = True
            print(f'Impact detected at t = {veh.model.t[i]} seconds')
            print(f'i: {i}')
            if impact_type == 'IMPC':
                vehicle_list, impc_energy = impc(i, crush_data.impactp_veh2x[i], crush_data.impactp_veh2y[i], vehicle_list=vehicle_list,
                                                 sim_defaults=sim_defaults)  # run impc model - create inputs using vehicle class
                veh.model.Fx[i] = 0
                veh.model.Fy[i] = 0
                veh.model.Mz[i] = 0
                impc_complete = True  # <- only run IMPC model once
            elif impact_type == 'SS':
                vehicle_list = ss(vehicle_list, crush_data, kmutual, vehicle_mu, i)
            else:
                print(f'impact_type {impact_type} is not defined - no impact forces generated')
                veh.model.Fx[i] = 0
                veh.model.Fy[i] = 0
                veh.model.Mz[i] = 0


    return vehicle_list, crush_data



"""
functions for plotting data related to vehicle kinematics for multiple vehicles
"""
import math
import plotly.graph_objects as go
from plotly.subplots import make_subplots
import plotly.io as pio
pio.renderers.default = "browser"


# TODO: create input for figure size - loads from "defaults" folder?
aspect_ratio = 16 / 9
font_size = 24
tick_font_size = 22
line_width = 2.5
figure_size_wide = (1600, 1200)
figure_size_tall = (1600, 1600)


def compare_kinematics(model1, model2, name1, name2):
    fig = make_subplots(rows=2, cols=1,
                        shared_xaxes=True,
                        vertical_spacing=0.07)

    # CG translations
    fig.add_trace(go.Scatter(x=model1.t, y=model1.Dx,
                             mode='lines',
                             name=f'{name1} - X',
                             line=dict(color='rgb(0, 255, 0)', width=line_width)),
                  row=1, col=1)
    fig.add_trace(go.Scatter(x=model2.t, y=model2.Dx,
                             mode='lines',
                             name=f'{name2} - X',
                             line=dict(color='rgb(0, 255, 0)', width=line_width, dash='dash')),
                  row=1, col=1)

    fig.add_trace(go.Scatter(x=model1.t, y=model1.Dy,
                             mode='lines',
                             name=f'{name1} - Y',
                             line=dict(color='rgb(0, 0, 255)', width=line_width)),
                  row=1, col=1)
    fig.add_trace(go.Scatter(x=model2.t, y=model2.Dy,
                             mode='lines',
                             name=f'{name2} - Y',
                             line=dict(color='rgb(0, 0, 255)', width=line_width, dash='dash')),
                  row=1, col=1)
    # Heading angle
    fig.add_trace(go.Scatter(x=model1.t, y=model1.theta_deg,
                             mode='lines',
                             name=f'{name1} - heading',
                             line=dict(color='rgb(0, 0, 0)', width=line_width)),
                  row=2, col=1)
    fig.add_trace(go.Scatter(x=model2.t, y=model2.theta_deg,
                             mode='lines',
                             name=f'{name2} - heading',
                             line=dict(color='rgb(0, 0, 0)', width=line_width, dash='dash')),
                  row=2, col=1)

    fig.update_layout(
        legend=dict(orientation="v", yanchor='top', y=.95, xanchor='left', x=1.05),
        autosize=False,
        width=figure_size_wide[0],
        height=figure_size_wide[1],
        template='plotly_white',
        yaxis=dict(showgrid=False),
        font=dict(family='Arial', size=font_size, color='black'))

    fig.update_xaxes(showgrid=False, title_text='Time (s)', tickfont=dict(size=tick_font_size), row=2, col=1)
    fig.update_xaxes(showgrid=False, title_text='', tickfont=dict(size=tick_font_size), row=1, col=1)
    fig.update_yaxes(showgrid=False, title_text='Displacement (ft)', tickfont=dict(size=tick_font_size), row=1, col=1)
    fig.update_yaxes(showgrid=False, title_text='Heading Angle (deg)', tickfont=dict(size=tick_font_size), row=2, col=1)
    fig.update_xaxes(showline=True, linewidth=1, linecolor='black', ticks="outside",
                     tickwidth=1, tickcolor='black', ticklen=10, zeroline=False)
    fig.update_yaxes(showline=True, linewidth=1, linecolor='black', ticks="outside",
                     tickwidth=1, tickcolor='black', ticklen=10, zeroline=False)
    fig.show()

    """
    velocity
    slip angle
    """
    fig = make_subplots(rows=2, cols=1,
                        shared_xaxes=True,
                        vertical_spacing=0.07)

    # CG Velocity
    fig.add_trace(go.Scatter(x=model1.t, y=model1.Vx / 1.46667,
                             mode='lines',
                             name=f'{name1} - X',
                             line=dict(color='rgb(0, 255, 0)', width=line_width)),
                  row=1, col=1)
    fig.add_trace(go.Scatter(x=model2.t, y=model2.Vx / 1.46667,
                             mode='lines',
                             name=f'{name2} - X',
                             line=dict(color='rgb(0, 255, 0)', width=line_width, dash='dash')),
                  row=1, col=1)

    fig.add_trace(go.Scatter(x=model1.t, y=model1.Vy / 1.46667,
                             mode='lines',
                             name=f'{name1} - Y',
                             line=dict(color='rgb(0, 0, 255)', width=line_width)),
                  row=1, col=1)
    fig.add_trace(go.Scatter(x=model2.t, y=model2.Vy / 1.46667,
                             mode='lines',
                             name=f'{name2} - Y',
                             line=dict(color='rgb(0, 0, 255)', width=line_width, dash='dash')),
                  row=1, col=1)
    # Slip angle - compare to beta_deg
    fig.add_trace(go.Scatter(x=model1.t, y=model1.phi_deg,
                             mode='lines',
                             name=f'{name1} - slip angle',
                             line=dict(color='rgb(0, 0, 0)', width=line_width)),
                  row=2, col=1)
    fig.add_trace(go.Scatter(x=model2.t, y=model2.phi_deg,
                             mode='lines',
                             name=f'{name2} - slip angle',
                             line=dict(color='rgb(0, 0, 0)', width=line_width, dash='dash')),
                  row=2, col=1)

    fig.update_layout(
        legend=dict(orientation="h", yanchor='top', y=1.1, xanchor='left', x=0.01),
        autosize=False,
        width=figure_size_tall[0],
        height=figure_size_tall[1],
        template='plotly_white',
        yaxis=dict(showgrid=False),
        font=dict(family='Arial', size=font_size, color='black'))

    fig.update_xaxes(showgrid=False, title_text='Time (s)', row=2, col=1)
    fig.update_xaxes(showgrid=False, title_text='', row=1, col=1)
    fig.update_yaxes(showgrid=False, title_text='Velocity (mph)', row=1, col=1)
    fig.update_yaxes(showgrid=False, title_text='Slip Angle (deg)', row=2, col=1)
    fig.update_xaxes(showline=True, linewidth=1, linecolor='black', ticks="outside",
                     tickwidth=1, tickcolor='black', ticklen=10, zeroline=False)
    fig.update_yaxes(showline=True, linewidth=1, linecolor='black', ticks="outside",
                     tickwidth=1, tickcolor='black', ticklen=10, zeroline=False)
    fig.show()


    fig = make_subplots(rows=4, cols=1,
                        shared_xaxes=True,
                        vertical_spacing=0.05)

    """
    rightward tire forces
    """
    fig.add_trace(go.Scatter(x=model1.t, y=model1.lf_fy,
                             mode='lines',
                             name=f'LF - {name1}',
                             line=dict(color='rgb(0, 0, 255)', width=line_width)),
                  row=1, col=1)
    fig.add_trace(go.Scatter(x=model2.t, y=model2.lf_fy,
                             mode='lines',
                             name=f'LF - {name2}',
                             line=dict(color='rgb(0, 0, 255)', width=line_width, dash='dash')),
                  row=1, col=1)
    fig.add_trace(go.Scatter(x=model1.t, y=model1.rf_fy,
                             mode='lines',
                             name=f'RF - {name1}',
                             line=dict(color='rgb(0, 255, 0)', width=line_width)),
                  row=2, col=1)
    fig.add_trace(go.Scatter(x=model2.t, y=model2.rf_fy,
                             mode='lines',
                             name=f'RF - {name2}',
                             line=dict(color='rgb(0, 255, 0)', width=line_width, dash='dash')),
                  row=2, col=1)
    fig.add_trace(go.Scatter(x=model1.t, y=model1.rr_fy,
                             mode='lines',
                             name=f'RR - {name1}',
                             line=dict(color='rgb(153, 0, 204)', width=line_width)),
                  row=3, col=1)
    fig.add_trace(go.Scatter(x=model2.t, y=model2.rr_fy,
                             mode='lines',
                             name=f'RR - {name2}',
                             line=dict(color='rgb(153, 0, 204)', width=line_width, dash='dash')),
                  row=3, col=1)
    fig.add_trace(go.Scatter(x=model1.t, y=model1.lr_fy,
                             mode='lines',
                             name=f'LR - {name1}',
                             line=dict(color='rgb(255, 102, 0)', width=line_width)),
                  row=4, col=1)
    fig.add_trace(go.Scatter(x=model2.t, y=model2.lr_fy,
                             mode='lines',
                             name=f'LR - {name2}',
                             line=dict(color='rgb(255, 102, 0)', width=line_width, dash='dash')),
                  row=4, col=1)

    fig.update_layout(
        legend=dict(orientation="v", yanchor='top', y=1.1, xanchor='left', x=1.01),
        autosize=False,
        width=figure_size_tall[0],
        height=figure_size_tall[1],
        title='Tire Rightward Forces',
        template='plotly_white',
        xaxis=dict(showgrid=False),
        yaxis=dict(showgrid=False),
        font=dict(family='Arial', size=font_size, color='black'))

    fig.update_xaxes(showgrid=False, title_text='Time (s)', row=4, col=1,
                     showline=True, linewidth=1, linecolor='black', ticks="outside",
                     tickwidth=1, tickcolor='black', ticklen=10, zeroline=False)
    fig.update_xaxes(showgrid=False, title_text='', row=1, col=1,
                     showline=True, linewidth=1, linecolor='black', ticks="outside",
                     tickwidth=1, tickcolor='black', ticklen=10, zeroline=False)
    fig.update_xaxes(showgrid=False, title_text='', row=2, col=1,
                     showline=True, linewidth=1, linecolor='black', ticks="outside",
                     tickwidth=1, tickcolor='black', ticklen=10, zeroline=False)
    fig.update_xaxes(showgrid=False, title_text='', row=3, col=1,
                     showline=True, linewidth=1, linecolor='black', ticks="outside",
                     tickwidth=1, tickcolor='black', ticklen=10, zeroline=False)
    fig.update_yaxes(showgrid=False, title_text='LF - Rightward Force (lb)', row=1, col=1,
                     showline=True, linewidth=1, linecolor='black', ticks="outside",
                     tickwidth=1, tickcolor='black', ticklen=10, zeroline=False)
    fig.update_yaxes(showgrid=False, title_text='RF - Rightward Force (lb)', row=2, col=1,
                     showline=True, linewidth=1, linecolor='black', ticks="outside",
                     tickwidth=1, tickcolor='black', ticklen=10, zeroline=False)
    fig.update_yaxes(showgrid=False, title_text='RR - Rightward Force (lb)', row=3, col=1,
                     showline=True, linewidth=1, linecolor='black', ticks="outside",
                     tickwidth=1, tickcolor='black', ticklen=10, zeroline=False)
    fig.update_yaxes(showgrid=False, title_text='LR - Rightward Force (lb)', row=4, col=1,
                     showline=True, linewidth=1, linecolor='black', ticks="outside",
                     tickwidth=1, tickcolor='black', ticklen=10, zeroline=False)
    fig.show()

    fig = make_subplots(rows=4, cols=1,
                        shared_xaxes=True,
                        vertical_spacing=0.05)

    """
    slip angles
    """
    fig.add_trace(go.Scatter(x=model1.t, y=model1.lf_alpha * 180 / math.pi,
                             mode='lines',
                             name=f'LF - {name1}',
                             line=dict(color='rgb(0, 0, 255)', width=2)),
                  row=1, col=1)
    fig.add_trace(go.Scatter(x=model2.t, y=model2.lf_alpha * -1,
                             mode='lines',
                             name=f'LF - {name2}',
                             line=dict(color='rgb(0, 0, 255)', width=line_width, dash='dash')),
                  row=1, col=1)
    fig.add_trace(go.Scatter(x=model1.t, y=model1.rf_alpha * 180 / math.pi,
                             mode='lines',
                             name=f'RF - {name1}',
                             line=dict(color='rgb(0, 255, 0)', width=line_width)),
                  row=2, col=1)
    fig.add_trace(go.Scatter(x=model2.t, y=model2.rf_alpha * -1,
                             mode='lines',
                             name=f'RF - {name2}',
                             line=dict(color='rgb(0, 255, 0)', width=line_width, dash='dash')),
                  row=2, col=1)
    fig.add_trace(go.Scatter(x=model1.t, y=model1.rr_alpha * 180 / math.pi,
                             mode='lines',
                             name=f'RR - {name1}',
                             line=dict(color='rgb(153, 0, 204)', width=2)),
                  row=3, col=1)
    fig.add_trace(go.Scatter(x=model2.t, y=model2.rr_alpha * -1,
                             mode='lines',
                             name=f'RR - {name2}',
                             line=dict(color='rgb(153, 0, 204)', width=line_width, dash='dash')),
                  row=3, col=1)
    fig.add_trace(go.Scatter(x=model1.t, y=model1.lr_alpha * 180 / math.pi,
                             mode='lines',
                             name=f'LR - {name1}',
                             line=dict(color='rgb(255, 102, 0)', width=2)),
                  row=4, col=1)
    fig.add_trace(go.Scatter(x=model2.t, y=model2.lr_alpha * -1,
                             mode='lines',
                             name=f'LR - {name2}',
                             line=dict(color='rgb(255, 102, 0)', width=line_width, dash='dash')),
                  row=4, col=1)

    fig.update_layout(
        legend=dict(orientation="v", yanchor='top', y=1.1, xanchor='left', x=1.01),
        autosize=False,
        width=figure_size_tall[0],
        height=figure_size_tall[1],
        title='Tire Slip Angles',
        template='plotly_white',
        xaxis=dict(showgrid=False),
        yaxis=dict(showgrid=False),
        font=dict(family='Arial', size=font_size, color='black'))

    fig.update_xaxes(showgrid=False, title_text='Time (s)', row=4, col=1,
                     showline=True, linewidth=1, linecolor='black', ticks="outside",
                     tickwidth=1, tickcolor='black', ticklen=10, zeroline=False)
    fig.update_xaxes(showgrid=False, title_text='', row=1, col=1,
                     showline=True, linewidth=1, linecolor='black', ticks="outside",
                     tickwidth=1, tickcolor='black', ticklen=10, zeroline=False)
    fig.update_xaxes(showgrid=False, title_text='', row=2, col=1,
                     showline=True, linewidth=1, linecolor='black', ticks="outside",
                     tickwidth=1, tickcolor='black', ticklen=10, zeroline=False)
    fig.update_xaxes(showgrid=False, title_text='', row=3, col=1,
                     showline=True, linewidth=1, linecolor='black', ticks="outside",
                     tickwidth=1, tickcolor='black', ticklen=10, zeroline=False)
    fig.update_yaxes(showgrid=False, title_text='LF - Slip Angle (deg)', row=1, col=1,
                     showline=True, linewidth=1, linecolor='black', ticks="outside",
                     tickwidth=1, tickcolor='black', ticklen=10, zeroline=False)
    fig.update_yaxes(showgrid=False, title_text='RF - Slip Angle (deg)', row=2, col=1,
                     showline=True, linewidth=1, linecolor='black', ticks="outside",
                     tickwidth=1, tickcolor='black', ticklen=10, zeroline=False)
    fig.update_yaxes(showgrid=False, title_text='RR - Slip Angle (deg)', row=3, col=1,
                     showline=True, linewidth=1, linecolor='black', ticks="outside",
                     tickwidth=1, tickcolor='black', ticklen=10, zeroline=False)
    fig.update_yaxes(showgrid=False, title_text='LR - Slip Angle (deg)', row=4, col=1,
                     showline=True, linewidth=1, linecolor='black', ticks="outside",
                     tickwidth=1, tickcolor='black', ticklen=10, zeroline=False)
    fig.show()

    # acceleration
    """"
    fig = make_subplots(rows=2, cols=1,
                        shared_xaxes=True,
                        vertical_spacing=0.07)

    # acceleration
    fig.add_trace(go.Scatter(x=model1.t, y=model1.au / 32.2,
                             mode='lines',
                             name=f'{name1} - ax',
                             line=dict(color='rgb(0, 255, 0)', width=2)),
                  row=1, col=1)
    fig.add_trace(go.Scatter(x=model2.t, y=model2.ax / 32.2,
                             mode='lines',
                             name=f'{name2} - ax',
                             line=dict(color='rgb(0, 255, 0)', width=2, dash='dash')),
                  row=1, col=1)

    fig.add_trace(go.Scatter(x=model1.t, y=model1.av / 32.2,
                             mode='lines',
                             name=f'{name1} - ay',
                             line=dict(color='rgb(0, 0, 255)', width=2)),
                  row=2, col=1)
    fig.add_trace(go.Scatter(x=model2.t, y=model2.ay / 32.2,
                             mode='lines',
                             name=f'{name2} - ay',
                             line=dict(color='rgb(0, 0, 255)', width=2, dash='dash')),
                  row=2, col=1)

    fig.update_layout(
        legend=dict(orientation="h", yanchor='top', y=1, xanchor='left', x=0.01),
        autosize=False,
        width=900,
        height=900,
        title='Acceleration',
        template='plotly_white',
        xaxis=dict(showgrid=False),
        yaxis=dict(showgrid=False),
        font=dict(family='Arial', size=14, color='black'))

    fig.update_xaxes(showgrid = False, title_text = '', row = 1, col = 1,
                     showline=True, linewidth=1, linecolor='black', ticks="outside",
                     tickwidth=1, tickcolor='black', ticklen=10, zeroline=False)
    fig.update_xaxes(showgrid = False, title_text = 'Time (s)', row = 2, col = 1,
                     showline=True, linewidth=1, linecolor='black', ticks="outside",
                     tickwidth=1, tickcolor='black', ticklen=10, zeroline=False)
    fig.update_yaxes(showgrid=False, title_text='Forward Accel (g)', row=1, col=1,
                     showline=True, linewidth=1, linecolor='black', ticks="outside",
                     tickwidth=1, tickcolor='black', ticklen=10, zeroline=False)
    fig.update_yaxes(showgrid=False, title_text='Rightward Accel (g)', row=2, col=1,
                     showline=True, linewidth=1, linecolor='black', ticks="outside",
                     tickwidth=1, tickcolor='black', ticklen=10, zeroline=False)
    fig.show()
    """

    """
    vertical tire forces
    """

    fig = make_subplots(rows=4, cols=1,
                        shared_xaxes=True,
                        vertical_spacing=0.05)

    fig.add_trace(go.Scatter(x=model1.t, y=model1.lf_fz,
                             mode='lines',
                             name=f'LF - {name1}',
                             line=dict(color='rgb(0, 0, 255)', width=2)), row=1, col=1)
    fig.add_trace(go.Scatter(x=model2.t, y=model2.lf_fz * 1000,
                             mode='lines',
                             name=f'LF - {name2}',
                             line=dict(color='rgb(0, 0, 255)', width=line_width, dash='dash')), row=1, col=1)
    fig.add_trace(go.Scatter(x=model1.t, y=model1.rf_fz,
                             mode='lines',
                             name=f'RF - {name1}',
                             line=dict(color='rgb(0, 255, 0)', width=line_width)), row=2, col=1)
    fig.add_trace(go.Scatter(x=model2.t, y=model2.rf_fz * 1000,
                             mode='lines',
                             name=f'RF - {name2}',
                             line=dict(color='rgb(0, 255, 0)', width=line_width, dash='dash')), row=2, col=1)
    fig.add_trace(go.Scatter(x=model1.t, y=model1.rr_fz,
                             mode='lines',
                             name=f'RR - {name1}',
                             line=dict(color='rgb(153, 0, 204)', width=line_width)), row=3, col=1)
    fig.add_trace(go.Scatter(x=model2.t, y=model2.rr_fz * 1000,
                             mode='lines',
                             name=f'RR - {name2}',
                             line=dict(color='rgb(153, 0, 204)', width=line_width, dash='dash')), row=3, col=1)
    fig.add_trace(go.Scatter(x=model1.t, y=model1.lr_fz,
                             mode='lines',
                             name=f'LR - {name1}',
                             line=dict(color='rgb(255, 102, 0)', width=line_width)), row=4, col=1)
    fig.add_trace(go.Scatter(x=model2.t, y=model2.lr_fz * 1000,
                             mode='lines',
                             name=f'LR - {name2}',
                             line=dict(color='rgb(255, 102, 0)', width=line_width, dash='dash')), row=4, col=1)

    fig.update_layout(
        legend=dict(orientation="v", yanchor='top', y=1.1, xanchor='left', x=1.01),
        autosize=False,
        width=figure_size_tall[0],
        height=figure_size_tall[1],
        title='Vertical Tire Forces',
        template='plotly_white',
        xaxis=dict(showgrid=False),
        yaxis=dict(showgrid=False),
        font=dict(family='Arial', size=font_size, color='black'))

    fig.update_xaxes(showgrid=False, title_text='Time (s)', row=4, col=1,
                     showline=True, linewidth=1, linecolor='black', ticks="outside",
                     tickwidth=1, tickcolor='black', ticklen=10, zeroline=False)
    fig.update_xaxes(showgrid=False, title_text='', row=1, col=1,
                     showline=True, linewidth=1, linecolor='black', ticks="outside",
                     tickwidth=1, tickcolor='black', ticklen=10, zeroline=False)
    fig.update_xaxes(showgrid=False, title_text='', row=2, col=1,
                     showline=True, linewidth=1, linecolor='black', ticks="outside",
                     tickwidth=1, tickcolor='black', ticklen=10, zeroline=False)
    fig.update_xaxes(showgrid=False, title_text='', row=3, col=1,
                     showline=True, linewidth=1, linecolor='black', ticks="outside",
                     tickwidth=1, tickcolor='black', ticklen=10, zeroline=False)
    fig.update_yaxes(showgrid=False, title_text='LF - Force (lb)', row=1, col=1,
                     showline=True, linewidth=1, linecolor='black', ticks="outside",
                     tickwidth=1, tickcolor='black', ticklen=10, zeroline=False)
    fig.update_yaxes(showgrid=False, title_text='RF - Force (lb)', row=2, col=1,
                     showline=True, linewidth=1, linecolor='black', ticks="outside",
                     tickwidth=1, tickcolor='black', ticklen=10, zeroline=False)
    fig.update_yaxes(showgrid=False, title_text='RR - Force (lb)', row=3, col=1,
                     showline=True, linewidth=1, linecolor='black', ticks="outside",
                     tickwidth=1, tickcolor='black', ticklen=10, zeroline=False)
    fig.update_yaxes(showgrid=False, title_text='LR - Force (lb)', row=4, col=1,
                     showline=True, linewidth=1, linecolor='black', ticks="outside",
                     tickwidth=1, tickcolor='black', ticklen=10, zeroline=False)
    fig.show()



import os
import pickle
import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
from scipy import integrate
from pycrash.functions.CFCFilter import cfcfilt


def process_loadcell_asii(test_num, path, row_list, num_columns, impact_speed, english=True):
    """
    download the NHTSA asii data into a single directory
    test_num -> NHTSA test number
    path -> path to directory containing asii files, including the .EV5 file
    row_list -> list containing row names (alphabetical) to be including in processing
    num_columns -> number of columns (integer)
    impact_velocity - barrier impact speed [kph]
    outputs will be saved in same directory
    defaults to English units, can be set to False
    will print cells that are missing data
    """

    format_file = f'v{test_num}.EV5'  # file containing channel info
    if english:
        accel_convert = 32.2
        impact_velocity = impact_speed * 0.911344  # convert kph to fps
        force_convert = 0.224809
    else:
        accel_convert = 9.81
        impact_velocity = impact_speed * 0.277778  # convert kph to m/s
        force_convert = 1

    # find lines with instrumentation info
    with open(os.path.join(path, format_file)) as myFile:
        for num, line in enumerate(myFile, 1):
            if 'INSTRUMENTATION' in line:
                print(f'Instrumentation data found at line: {num}')
                instStart = num
            elif '- END -' in line:
                print(f'Instrumentation data found at line: {num}')
                instEnd = num

    # find info for barrier data Fx
    test_data = {}  # <- dictionary of load cell data
    with open(os.path.join(path, format_file)) as myFile:
        for num, line in enumerate(myFile, 1):
            # search for load cell barrier channels
            if (num > instStart) & (num < instEnd) & ('Fx' in line) & ('BARRIER' in line):
                # print(line.split('|'))
                channel_name = line.split('|')[-1].replace('\n', '')
                channel_num = int(line.split('|')[1])
                lc_row = channel_name[8]
                lc_col = int(channel_name[10:12])
                print(f'Channel: {channel_name} #{channel_num} at row: {lc_row}, col: {lc_col}')
                test_data[channel_name] = {'type': 'LC',
                                           'num': channel_num,
                                           'row': lc_row,
                                           'col': lc_col,
                                           'fileName': f'v{test_num}.{channel_num}'}

            # search for vehicle crossmember acceleration data Ax
            elif (num > instStart) & (num < instEnd) & ('REAR SEAT CROSSMEMBER X' in line):
                channel_name = line.split('|')[-1].replace('\n', '')
                channel_num = int(line.split('|')[1])
                print(f'Channel: {channel_name} #{channel_num}')
                if channel_num > 99:
                    fileName = f'v{test_num}.{channel_num}'
                elif (channel_num <= 99) & (channel_num > 9):
                    fileName = f'v{test_num}.0{channel_num}'
                elif channel_num <= 9:
                    fileName = f'v{test_num}.00{channel_num}'
                test_data[channel_name] = {'type': 'Accel',
                                           'name': channel_name,
                                           'num': channel_num,
                                           'loc': 'Crossmember',
                                           'dir': 'x',
                                           'fileName': fileName}

    # load data and load cell data for desired rows
    for key, value in test_data.items():
        if value['type'] == 'LC':
            if (value['row'] in row_list) & ('No valid data' in key):
                print(f'No valid data: {key}')
                test_data[key]['colName'] = f'{value["row"]}_{value["col"]}'
                test_data[key]['time'] = timeData
                test_data[key]['rawforce'] = [0] * len(timeData)  # set force to zero if missing
                test_data[key]['rawforce'] = [0] * len(timeData)  # set force to zero if missing
                test_data[key]['force'] = [0] * len(timeData)
            elif (value['row'] in row_list) & ('No valid data' not in key):
                print(f'loading file {value["fileName"]}')
                # load file with numpy
                data_array = np.loadtxt(os.path.join(path, value["fileName"]), delimiter='\t')
                all_time = list(data_array[:, 0])
                all_data = list(data_array[:, 1])
                zero_time_index = all_time.index(0)
                time = all_time[zero_time_index:]
                data = all_data[zero_time_index:]
                test_data[key]['colName'] = f'{value["row"]}_{value["col"]}'
                test_data[key]['time'] = time
                test_data[key]['rawforce'] = [x * force_convert for x in data]
                test_data[key]['force'] = cfcfilt(60, [x * force_convert for x in data], time[1] - time[0])
                timeData = time
        elif value['type'] == 'Accel':
            data_array = np.loadtxt(os.path.join(path, value["fileName"]), delimiter='\t')
            all_time = list(data_array[:, 0])
            all_data = list(data_array[:, 1])
            zero_time_index = all_time.index(0)
            time = all_time[zero_time_index:]
            data = all_data[zero_time_index:]
            test_data[key]['colName'] = value['name']
            test_data[key]['time'] = time
            test_data[key]['rawAccel'] = data
            filtered_accel = cfcfilt(60, data, time[1] - time[0])
            test_data[key]['Accel'] = filtered_accel  # filtered acceleration [g]
            accel_ = [x * accel_convert for x in filtered_accel]
            velocity = integrate.cumtrapz(accel_, time, initial=0)
            velocity = [x + impact_velocity for x in velocity]
            test_data[key]['Velocity'] = velocity  # velocity [fps / m/s]
            test_data[key]['Displacement'] = integrate.cumtrapz(velocity, time, initial=0)

    # create dictionary of filtered force
    forceDict = {}
    forceDict['time'] = test_data[next(iter(test_data))]['time']
    for key, value in test_data.items():
        if value['type'] == 'LC':
            # print(value['colName'])
            forceDict[value['colName']] = value['force']  # filtered force data

    # convert to dataframe
    forceDF = pd.DataFrame.from_dict(forceDict)
    print(f'Column Names in Force data: {list(forceDF)}')
    # fix columns with missing data
    for label, content in forceDF.items():
        if sum(content) == 0:
            print(f'Column {label} = 0')
            average_list = [str(x) for x in input("Enter a list of column names to average to replace missing data: (example: C_3, C_5)").split(', ')]
            if len(average_list) == 1:
                print(f'Assigning missing column data to column {average_list[0]}')
                forceDF.loc[:, label] = forceDF.loc[:, average_list[0]].copy()
            else:
                forceDF.loc[:, label] = forceDF[average_list].mean(axis=1)

    column_list = list(np.arange(1, num_columns))
    for row in row_list:
        sum_columns = [row + "_" + str(s) for s in column_list]
        forceDF[f'{row}_Sum'] = forceDF[sum_columns].sum(axis=1)

    # get velocity / displacement data
    AccelDict = {}
    AccelDict['time'] = test_data[next(iter(test_data))]['time']
    for key, value in test_data.items():
        if value['type'] == 'Accel':
            print(value['colName'])
            AccelDict[value['colName']] = value['Accel']  # filtered acceleration [g]
            AccelDict[f"{value['colName']} Velocity"] = value['Velocity']  # velocity [fps, mps]
            AccelDict[f"{value['colName']} Disp"] = value['Displacement']  # displacement [ft, m]

    AccelDF = pd.DataFrame.from_dict(AccelDict)

    # %% Combined data by columns -
    # sum by row
    rowForce = {}
    rowForce['time'] = test_data[next(iter(test_data))]['time']
    rowSum = [0] * len(test_data[next(iter(test_data))]['time'])
    currentRow = 'K'
    for key, value in test_data.items():
        if value['type'] == 'LC':
            if value['row'] == currentRow:
                rowSum = [a + b for a, b in zip(rowSum, value['force'])]
                rowForce[value['row']] = rowSum
            else:
                rowSum = [0] * len(test_data[next(iter(test_data))]['time'])
                rowSum = [a + b for a, b in zip(rowSum, value['force'])]
                rowForce[value['row']] = rowSum
                currentRow = value['row']

    RowForcedf = pd.DataFrame.from_dict(rowForce)
    # merge summed forces and displacement df
    Fdx = pd.merge(RowForcedf, AccelDF, how='left', on='time')
    Fdx.to_pickle(os.path.join(path, f'RowSum_ProcessedNHTSA_Test_{test_num}.pkl'))

    # all force data with accel / displacement df
    test_data_accel = pd.merge(forceDF, AccelDF, how='left', on='time')
    test_data_accel.to_pickle(os.path.join(path, f'ProcessedNHTSATestData_{test_num}.pkl'))

    print("Processing Complete - Files Saved to Path Provided")



import numpy as np

def impc_calcs(veh1, veh2, poi_veh2x, poi_veh2y, sim_inputs):
    cor = sim_inputs['cor']
    cof = sim_inputs['cof']
    impact_norm_deg = sim_inputs['impact_norm_deg']
    # heading angle [rad] of tangent impact direction in global frame
    theta_c = (veh1.head_angle * np.pi / 180) + (impact_norm_deg * np.pi / 180) + (90 / 180 * np.pi)
    print(f'theta c: {theta_c * 180 / np.pi:0.2f} [deg]')

    # carpenter + welcher model
    # get cosine / sine results for coordinate transformation to the normal - tangent axis
    c1 = np.cos(veh1.head_angle * np.pi / 180 - theta_c)
    s1 = np.sin(veh1.head_angle * np.pi / 180 - theta_c)
    c2 = np.cos(veh2.head_angle * np.pi / 180 - theta_c)
    s2 = np.sin(veh2.head_angle * np.pi / 180 - theta_c)

    # translate distances to n-t frame
    dt1 = c1*(veh1.pimpact_x) - s1*(veh1.pimpact_y)
    dn1 = s1*(veh1.pimpact_x) + c1*(veh1.pimpact_y)
    dt2 = c2*(poi_veh2x) - s2*(poi_veh2y)
    dn2 = s2*(poi_veh2x) + c2*(poi_veh2y)

    # translate velocities to n-t frame
    vt1 = c1*veh1.vx_initial - s1*veh1.vy_initial
    vn1 = s1*veh1.vx_initial + c1*veh1.vy_initial
    vt2 = c2*veh2.vx_initial - s2*veh2.vy_initial
    vn2 = s2*veh2.vx_initial + c2*veh2.vy_initial

    # pre-impact POI velocities (Equations 1)
    vct1 = vt1 - dn1 * veh1.omega_z * np.pi / 180
    vct2 = vt2 - dn2 * veh2.omega_z * np.pi / 180
    vct21 = vct2 - vct1

    vcn1 = vn1 + dt1 * veh1.omega_z * np.pi / 180
    vcn2 = vn2 + dt2 * veh2.omega_z * np.pi / 180
    vcn21 = vcn2 - vcn1

    A11 = (1 / (veh1.weight / 32.2) + 1 / (veh2.weight / 32.2) + dn1**2 / veh1.izz + dn2**2 / veh2.izz)
    A12 = (dt1 * dn1 / veh1.izz + dt2 * dn2 / veh2.izz)
    A22 = (1 / (veh1.weight / 32.2) + 1 / (veh2.weight / 32.2) + dt1**2 / veh1.izz + dt2**2 / veh2.izz)
    A = (A11 * A22 - A12 * A12)

    # collision impulse -
    pt = (1+cor) * (A22 * vct21 + A12 * vcn21) / A
    pn = (1+cor) * (A12 * vct21 + A11 * vcn21) / A

    p_ratio = abs(pt/pn)   # equation 10
    alpha = 1

    if p_ratio > cof:
            print('Sliding Condition')
            pn_s = (1+cor) * vcn21 / (A22 - alpha * cof * A12)
            pt_s = alpha * cof * pn_s

            if pt * pt_s > 0:           # test for sliding direction
                pt = pt_s               # alpha = 1 is correct, keep initial try
                pn = pn_s
            else:
                alpha = -1
                pn = (1+cor) * vcn21 / (A22 - alpha * cof * A12)   # sliding in wrong direction, alpha = -1
                pt = alpha * cof * pn
    else:
        print ('No Sliding')                                                      # no sliding, original pt, pn are correct

    if pn * dn1 < 0:
        print('Compression')
    else:
        print('Tension')

    # change in velocity in collision frame
    # Equation 16 - termination of restitution

    dvt1 = pt / (veh1.weight/32.2)
    dvn1 = pn / (veh1.weight/32.2)
    doz1 = (-1*dn1 * pt + dt1 * pn) / veh1.izz

    dvt2 = -1 * pt / (veh2.weight/32.2)
    dvn2 = -1 * pn / (veh2.weight/32.2)
    doz2 = (dn2 * pt - dt2 * pn) / veh2.izz

    # transform back to local vehicle coordinate system
    # delta-Vs
    dvx1 = (c1*dvt1 + s1*dvn1)
    dvy1 = (-1*s1*dvt1 + c1*dvn1)
    dveh1 = np.sqrt(dvx1**2 + dvy1**2)
    doz1_deg = doz1 * 180 / np.pi

    dvx2 = (c2*dvt2 + s2*dvn2)
    dvy2 = (-1*s2*dvt2 + c2*dvn2)
    dveh2 = np.sqrt(dvx2**2 + dvy2**2)
    doz2_deg = doz2 * 180 / np.pi

    # post impact speeds
    vx1_ = veh1.vx_initial + dvx1
    vy1_ = veh1.vy_initial + dvy1
    veh1_ = np.sqrt(vx1_**2 + vy1_**2)
    oz_deg1_ = veh1.omega_z + doz1_deg
    oz_rad1_ = oz_deg1_ * np.pi / 180

    vx2_ = veh2.vx_initial + dvx2
    vy2_ = veh2.vy_initial + dvy2
    veh2_ = np.sqrt(vx2_**2 + vy2_**2)
    oz_deg2_ = veh2.omega_z + doz2_deg
    oz_rad2_ = oz_deg2_ * np.pi / 180

    # calculations for energy dissipated
    vmt1 = vt1 + dvt1 / (1 + cor)
    omgm1 = veh1.omega_z + doz1 / (1 + cor)
    vmt2 = vt2 + dvt2 / (1 + cor)
    omgm2 = veh2.omega_z + doz2 / (1 + cor)

    vmct21 = vmt2 - vmt1 - dn2 * omgm2 + dn1 * omgm1

    if vmct21 * vct21 < 0:
        print('Reverse Slide')
    else:
        print('Forward Slide')

    vt1_ = vt1 + dvt1
    vt2_ = vt2 + dvt2
    vct21_ = vt2_ - dn2 * oz_rad2_ - vt1_ + dn1 * oz_rad1_

    vn1_ = vn1 + dvn1
    vn2_ = vn2 + dvn2
    vcn21_ = vn2_ + dt2 * oz_rad2_ - vn1_ - dt1 * oz_rad1_

    # dissipated energies
    ke_surface_dis = pt * vmct21
    ke_volume_dis = 0.5 * (1-cor) * (pt * (vct21 - vmct21) + pn * vcn21)
    ke_total_dis = ke_surface_dis + ke_volume_dis

    ke_volume_abs = 0.5 / (1+cor) * (pt * (vct21 - vmct21) + pn * vcn21)
    ke_total_abs = 0.5 / (1+cor) * (pt * (vct21 + vmct21) + pn * vcn21)

    t_effects_dis = 0.5 * pt * (vct21 + vct21_)
    n_effects_dis = 0.5 * pn * (vcn21 + vcn21_)
    tn_total_dis = t_effects_dis + n_effects_dis

    # speeds in mph -
    veh1_imp = {'vx': vx1_* 0.681818, 'vy': vy1_* 0.681818, 'oz_rad': oz_rad1_, 'oz_deg': oz_deg1_, 'dvx':dvx1 * 0.681818, 'dvy': dvy1 * 0.681818, 'dv':dveh1 * 0.681818}
    veh2_imp = {'vx': vx2_* 0.681818, 'vy': vy2_* 0.681818, 'oz_rad': oz_rad2_, 'oz_deg': oz_deg2_, 'dvx':dvx2 * 0.681818, 'dvy': dvy2 * 0.681818, 'dv':dveh2 * 0.681818}


    #veh1.impc_result = {'vx_post': vx1_, 'vy_post': vy1_, 'oz_rad_post': oz_rad1_, 'dvx':dvx1, 'dvy': dvy1, 'dv':dveh1}
    #veh2.impc_result = {'vx_post': vx2_, 'vy_post': vy2_, 'oz_rad_post': oz_rad2_, 'dvx':dvx2, 'dvy': dvy2, 'dv':dveh2}
    impc_energy = {'t_effects_dis': t_effects_dis, 'n_effects_dis':n_effects_dis, 'tn_total_dis':tn_total_dis}

    print(f'Impact Energy: {impc_energy}')
    return veh1_imp, veh2_imp
